#include "BookSection.h"
#include <sys/types.h>
#include <regex.h>
#include <iostream>
#include <fstream>
#include <assert.h>
#include "PorterStemmer.h"
#include "Config.h"
#include "Utils.h"
#include "Word.h"
#include <algorithm>

using namespace std;


Unigram_to_Count_map_t		BookSection::map_StemToSectionCount;
Unigram_to_Count_map_t		BookSection::map_WordToSectionCount;
Unigram_to_Count_map_t		BookSection::map_Stem_to_First_Occurrence_by_Section;
BookSection_list_t			BookSection::lst_AllSections;
BookSection*				BookSection::p_Root = NULL;
int							BookSection::i_TotalSections;



//======================================================
BookSection::BookSection ( void )
{
    p_Title = NULL;
    p_Parent = NULL;
    p_PreviousSection = NULL;
    p_NextSibling = NULL;
    p_PreviousSibling = NULL;
    i_Depth = -1;
}


//======================================================
BookSection::BookSection (const char* _zTitle, const char* _zBody)
{
    p_Title = NULL;
    p_Parent = NULL;
    p_PreviousSection = NULL;
    p_NextSibling = NULL;
    p_PreviousSibling = NULL;
    i_Depth = -1;

    BookSection::SetText (_zTitle, _zBody);
}


//======================================================
BookSection::~BookSection (void)
{
}


//======================================================
void BookSection::SetText (const char* _zTitle, const char* _zBody)
{
    p_Title = new Sentence (_zTitle);

    String sBody (_zBody);
	Utils::CleanText (sBody);

    list <String> lstSentenceText;
    Utils::SplitIntoSentences (sBody, lstSentenceText);
    
    ITERATE_COLLECTION (list <String>, lstSentenceText, ite)
	lst_Sentences.push_back (new Sentence (*ite));

    i_TotalUnigrams = 0;   
    long lSentencePosition = 0;
    ITERATE_COLLECTION (Sentence_list_t, lst_Sentences, iteSentence)
	{
	    Sentence* pSentence = *iteSentence;
	    
	    long lWordPosition = 0;
	    ITERATE_COLLECTION (Word_list_t, pSentence->lst_Words, iteWord)
		{
		    Word* pWord = *iteWord;
		    
		    map_Stem_to_First_Occurrence_by_Word.insert (make_pair (pWord->s_Stem, lWordPosition));
		    map_Stem_to_First_Occurrence_by_Sentence.insert (make_pair (pWord->s_Stem, lSentencePosition));
		    
		    Unigram_to_Count_map_t::iterator	iteCount;
		    
		    // stem counts...					
		    iteCount = map_Stem_to_Count.find (pWord->s_Stem);
		    if (iteCount == map_Stem_to_Count.end ())
				map_Stem_to_Count.insert (make_pair (pWord->s_Stem, 1));
		    else
				++ iteCount->second;
		    
		    // word counts...					
		    iteCount = map_Word_to_Count.find (pWord->s_Word);
		    if (iteCount == map_Word_to_Count.end ())
				map_Word_to_Count.insert (make_pair (pWord->s_Word, 1));
		    else
				++ iteCount->second;
		    
		    ++ lWordPosition;
		    ++i_TotalUnigrams;
		}
	    ++ lSentencePosition;
	}
}


//======================================================
Word_list_t BookSection::GetBodyWords (void)
{
    Word_list_t lstBodyWords;

    ITERATE_COLLECTION (Sentence_list_t, lst_Sentences, iteSentence)
	{
	    Sentence* pSentence = *iteSentence;
	    
	    ITERATE_COLLECTION (Word_list_t, pSentence->lst_Words, iteWord)
		lstBodyWords.push_back (*iteWord);
	}
	 
    return lstBodyWords;
}   


//-------------------------------------------------------------------
void BookSection::CreateUniqueWordList (void)
{
	typedef map <String, Word*>	UniqueWord_map_t;
	UniqueWord_map_t	mapUniqueWords;

	ITERATE_COLLECTION (Sentence_list_t, lst_Sentences, iteSentence)
	{
		Sentence* pSentence = *iteSentence;
		ITERATE_COLLECTION (Word_list_t, pSentence->lst_Words, iteWord)
		{
			Word* pWord = *iteWord;

			String sKey (pWord->s_Word);
			sKey << '|' << pWord->s_POSTag;

			mapUniqueWords.insert (make_pair (sKey, pWord));
		}
	}

	ITERATE_COLLECTION (UniqueWord_map_t, mapUniqueWords, ite)
		dq_UniqueWords.push_back (ite->second);

	mapUniqueWords.clear ();
}


//-------------------------------------------------------------------
Unigram_list_t BookSection::CapitalizeTitle (Word_list_t _lstTitleWords) 
{
	// Build Capitalization Map
    typedef map <String, int> Word_to_CapitalCount_map_t;
    typedef map <String, Word_to_CapitalCount_map_t> Word_to_CapitalWordAndCountMap_map_t;
    Word_to_CapitalWordAndCountMap_map_t map_Word_to_Capital;

    ITERATE_COLLECTION (Sentence_list_t, lst_Sentences, iteSentence) 
	{
		Unigram_list_t lstSentenceTokens;
		(*iteSentence)->s_Text.Split (lstSentenceTokens);

		ITERATE_COLLECTION (Unigram_list_t, lstSentenceTokens, iteWord) 
		{
			if ((lstSentenceTokens.begin () != iteWord) || 
				(lstSentenceTokens.begin () == iteWord && iteWord->IsUpper ())) 
			{
				String sLowerWord = *iteWord;
				sLowerWord.LowerCase ();

				if (map_Word_to_Capital.find (sLowerWord) == map_Word_to_Capital.end ())
				{
					Word_to_CapitalCount_map_t map_CapitalWord_to_Count; 
					map_Word_to_Capital.insert (make_pair (sLowerWord, map_CapitalWord_to_Count));
				}
		
				Word_to_CapitalWordAndCountMap_map_t::iterator	iteCapitalCount;
				iteCapitalCount = map_Word_to_Capital.find (sLowerWord);
				if (iteCapitalCount->second.find (*iteWord) == iteCapitalCount->second.end ())
					map_Word_to_Capital [sLowerWord] [*iteWord] = 0;

				++map_Word_to_Capital [sLowerWord] [*iteWord];
			}
		}
    }

    // Find max case for each word
    typedef map <String, String> map_Word_to_CapWord_t;
    map_Word_to_CapWord_t map_Word_to_CapWord;
    ITERATE_COLLECTION (Word_to_CapitalWordAndCountMap_map_t, map_Word_to_Capital, iteWordCap) 
	{
		String sWord = iteWordCap->first;
		String sMaxCase;
		int iMaxCaseCount = -1;
		ITERATE_COLLECTION (Word_to_CapitalCount_map_t, map_Word_to_Capital [sWord], iteCapCount) 
		{
			String sCasedWord = iteCapCount->first;
			int iCount = iteCapCount->second;
			
			if (iMaxCaseCount <= iCount)
			{
				iMaxCaseCount = iCount;
				sMaxCase = sCasedWord;
			}
		}
		map_Word_to_CapWord.insert (make_pair (sWord, sMaxCase));
    }

    // Case the title
    Unigram_list_t lstCapitalizedTitle;
    bool bFirstWord = true;
    ITERATE_COLLECTION (Word_list_t, _lstTitleWords, iteTitleWord) 
	{
		String sTitleWord = (*iteTitleWord)->s_Word;
		if (*iteTitleWord == Word::GetStartWord ()) 
		{
			lstCapitalizedTitle.push_back (sTitleWord);
			continue;
		}


		String sCapitalizedWord;
		if (map_Word_to_CapWord.find (sTitleWord) != map_Word_to_CapWord.end ())
			sCapitalizedWord = (const char*)map_Word_to_CapWord [sTitleWord];
		else
			sCapitalizedWord = sTitleWord;


		if (true == bFirstWord) 
		{
			if (sCapitalizedWord.IsLower ()) 
				sCapitalizedWord.Capitalize ();

			lstCapitalizedTitle.push_back (sCapitalizedWord);
			bFirstWord = false;
		} 
		else
			lstCapitalizedTitle.push_back (sCapitalizedWord);
	}
    
    return lstCapitalizedTitle;
}


//-------------------------------------------------------------------
bool BookSection::IsRoot (void) 
{
    return p_Parent == NULL;
}


//-------------------------------------------------------------------
void BookSection::GetDescendants (BookSection_list_t* _rlstSections)
{
	_rlstSections->push_back (this);

	ITERATE_COLLECTION (BookSection_list_t, lst_Children, ite) 
	{
		BookSection* pSection = *ite;
		pSection->GetDescendants (_rlstSections);
	}
}


//-------------------------------------------------------------------
BookSection* BookSection::GetRoot (void) 
{
    return BookSection::p_Root;
}


//-------------------------------------------------------------------
BookSection_list_t* BookSection::GetAllSections (void) 
{
	return &lst_AllSections;
}


//-------------------------------------------------------------------
void BookSection::AddChild (BookSection* _pChild)
{
    _pChild->p_Parent = this;

	_pChild->p_NextSibling = NULL;
	if (lst_Children.empty ())
		_pChild->p_PreviousSibling = NULL;
	else
	{
		BookSection* pPreviousSibling = lst_Children.back ();
		pPreviousSibling->p_NextSibling = _pChild;
		_pChild->p_PreviousSibling = pPreviousSibling;
	}

    lst_Children.push_back (_pChild);
}


//-------------------------------------------------------------------
void BookSection::BuildRootStructures (void)
{
    BookSection::p_Root = this;
    while (NULL != BookSection::p_Root->p_Parent)
		BookSection::p_Root = BookSection::p_Root->p_Parent;

	BookSection::p_Root->GetDescendants (&lst_AllSections);
	BookSection::p_Root->BuildFirstSectionWithStemMap ();
	BuildUnigramToSectionCounts ();
}


//-------------------------------------------------------------------
void BookSection::BuildUnigramToSectionCounts (void)
{
	BookSection_list_t* plstAllSection = GetAllSections ();
    ITERATE_COLLECTION_POINTER (BookSection_list_t, plstAllSection, iteSection) 
	{
		BookSection* pSection = *iteSection;
		ITERATE_COLLECTION (Unigram_to_Count_map_t, pSection->map_Stem_to_Count, iteStem)
		{
			Unigram_to_Count_map_t::iterator	ite;
			ite = map_StemToSectionCount.find (iteStem->first);
			if (ite == map_StemToSectionCount.end ())
				map_StemToSectionCount.insert (make_pair (iteStem->first, 1));
			else
				++ ite->second;
		}
		ITERATE_COLLECTION (Unigram_to_Count_map_t, pSection->map_Word_to_Count, iteWord)
		{
			Unigram_to_Count_map_t::iterator	ite;
			ite = map_WordToSectionCount.find (iteWord->first);
			if (ite == map_WordToSectionCount.end ())
				map_WordToSectionCount.insert (make_pair (iteWord->first, 1));
			else
				++ ite->second;
		}
    }
}


//-------------------------------------------------------------------
void BookSection::BuildFirstSectionWithStemMap (void) 
{
    if (false == IsRoot ()) 
	{
		cerr << "BuildFirstSectionWithStemMap: This should only be called on the root node" << endl;
		throw;
    }

    int iSectionNumber = 0;
    ITERATE_COLLECTION_POINTER (BookSection_list_t, GetAllSections (), iteSection) 
	{
		ITERATE_COLLECTION (Unigram_to_Count_map_t, (*iteSection)->map_Stem_to_First_Occurrence_by_Word, iteStem) 
		{
			String sStem = iteStem->first;
			if (map_Stem_to_First_Occurrence_by_Section.find (sStem) == map_Stem_to_First_Occurrence_by_Section.end ())
				map_Stem_to_First_Occurrence_by_Section.insert (make_pair (sStem, iSectionNumber));
		}
		++iSectionNumber;
    }
}


//-------------------------------------------------------------------
void BookSection::BuildNPCounts (void) 
{
    if (false == map_NP_to_Corpus_Count.empty ())
	{
		cerr << "NP map is empty!" << endl;
		return;
	}

    if (false == IsRoot ()) 
	{
		cerr << "BuildNPCounts: This should only be called on the root section" << endl;
		throw;
    }

	ITERATE_COLLECTION_POINTER (BookSection_list_t, GetAllSections (), iteSection) 
	{
		BookSection* pSection = *iteSection;
		ITERATE_COLLECTION (Sentence_list_t, pSection->lst_Sentences, iteSentence) 
		{
			Sentence* pSentence = *iteSentence;

			String sText (pSentence->s_Text);
			sText.LowerCase ();
			Ngram_list_t lstNgrams;

			Utils::ExtractNounPhrases (lstNgrams, &sText, &(pSentence->lst_Words));
			ITERATE_COLLECTION (Ngram_list_t, lstNgrams, iteNgram) 
			{
				String sNgram;
				Utils::WordListToNgramString (sNgram, (*iteNgram));

				Ngram_to_Count_map_t::iterator iteCount;
				iteCount = pSection->map_NP_to_Count.find (sNgram);
				if (iteCount == pSection->map_NP_to_Count.end ())
					pSection->map_NP_to_Count.insert (make_pair (sNgram, 1));
				else
					++ iteCount->second;
			}
			Utils::DeleteNgramList( lstNgrams );
		}

		Ngram_to_Count_map_t& mapNPCounts = pSection->map_NP_to_Count;
		ITERATE_COLLECTION (Ngram_to_Count_map_t, mapNPCounts, iteNPCount) 
		{
			const char* zNgram = iteNPCount->first;
			int iNgramCount = iteNPCount->second;

			Ngram_to_Count_map_t::iterator	iteCount;
			iteCount = map_NP_to_Corpus_Count.find (zNgram);
			if (iteCount == map_NP_to_Corpus_Count.end ())
				map_NP_to_Corpus_Count.insert (make_pair (zNgram, iNgramCount));
			else
				iteCount->second += iNgramCount;
		}
	}
	cout << "NPs : " << map_NP_to_Corpus_Count.size () << endl;
}


//-------------------------------------------------------------------
bool BookSection::HasUnigram (const char* _zUnigram, bool _bUseStem) 
{
	if (true == _bUseStem)
		return (map_Stem_to_Count.find (_zUnigram) != map_Stem_to_Count.end ());
	else
		return (map_Word_to_Count.find (_zUnigram) != map_Word_to_Count.end ());
}


//-------------------------------------------------------------------
int BookSection::GetUnigramOccurrence (const char* _zUnigram, bool _bUseStem) 
{
	Unigram_to_Count_map_t::iterator	ite;
    if (_bUseStem)
	{
		ite = map_Stem_to_Count.find (_zUnigram);
		if (ite == map_Stem_to_Count.end ())
			return 0;

		return ite->second;
	}
    else
	{
		ite = map_Word_to_Count.find (_zUnigram);
		if (ite == map_Word_to_Count.end ())
			return 0;

		return ite->second;
	}
}


//-------------------------------------------------------------------
int BookSection::GetNumberOfSectionsWithUnigram (const char* _zUnigram, bool _bUseStem) 
{
	if (true == _bUseStem)
		return map_StemToSectionCount [_zUnigram];
	else
		return map_WordToSectionCount [_zUnigram];
}


//-------------------------------------------------------------------
int BookSection::GetTotalUnigramOccurrences (void)
{
    return i_TotalUnigrams;
}


//-------------------------------------------------------------------
long BookSection::GetFirstOccurrenceByWord (const char* _zStem) 
{
    return map_Stem_to_First_Occurrence_by_Word [_zStem];
}


//-------------------------------------------------------------------
long BookSection::GetFirstOccurrenceBySentence (const char* _zStem) 
{
    return map_Stem_to_First_Occurrence_by_Sentence [_zStem];
}

//-------------------------------------------------------------------
long BookSection::GetFirstOccurrenceBySection (const char* _zStem) 
{
    if (false == IsRoot ()) 
	{
		cerr << "GetFirstOccurrenceBySection: This method can only be called on the root section" << endl;
		throw;
    }
    // Note: If the stem is not found in corpus, we return the last section
	Unigram_to_Count_map_t::iterator ite;
	ite = map_Stem_to_First_Occurrence_by_Section.find (_zStem);
	if (ite != map_Stem_to_First_Occurrence_by_Section.end ())
		return ite->second;

	return GetAllSections ()->size ();
} 


//-------------------------------------------------------------------
int BookSection::GetNPCount (const char* _zNP) 
{
	Ngram_to_Count_map_t::iterator	ite;
	ite = map_NP_to_Count.find (_zNP);
	if (ite == map_NP_to_Count.end ())
		return 0;

	return ite->second;
}


//-------------------------------------------------------------------
set<String> BookSection::GetBodyNPs (void) 
{
    set<String> set_NPs;
    ITERATE_COLLECTION (Ngram_to_Count_map_t, map_NP_to_Count, iteNPCount) 
		set_NPs.insert (iteNPCount->first);
 
    return set_NPs;
}


//-------------------------------------------------------------------
int BookSection::GetCorpusNPCount (const char* _zNP) 
{
    if (false == IsRoot ()) 
	{
		cerr << "GetCorpusNPCount: This method can only be called on the root section" << endl;
		throw;
    }

	Ngram_to_Count_map_t::iterator	ite;
	ite = map_NP_to_Corpus_Count.find (_zNP);
	if (ite == map_NP_to_Corpus_Count.end ())
		return 0;

	return ite->second;
}


//-------------------------------------------------------------------
bool BookSection::PreviousSiblingHasStem (const char* _zStem) 
{
	if (NULL == p_PreviousSibling)
		return false;
	return p_PreviousSibling->HasUnigram (_zStem, true);
}


//-------------------------------------------------------------------
bool BookSection::NextSiblingHasStem (const char* _zStem) 
{
	if (NULL == p_NextSibling)
		return false;
	return p_NextSibling->HasUnigram (_zStem, true);
}


//-------------------------------------------------------------------
Unigram_to_Count_map_t* BookSection::GetAllWords (void)
{
	return &map_WordToSectionCount;
}


//-------------------------------------------------------------------
int BookSection::GetDepth (void)
{
    if (-1 != i_Depth)
		return i_Depth;
    
    BookSection *pSection = this;
    while (NULL != pSection->p_Parent) 
	{
		pSection = pSection->p_Parent;
		++ i_Depth;
    }
    
    return i_Depth;
}


//-------------------------------------------------------------------
String BookSection::GetSectionXML (void)
{
	String sXML;
	sXML << "<section>\n";
	sXML << "<words>" << i_TotalUnigrams << "</words>\n";
	sXML << "<sentences>" << (long) lst_Sentences.size () << "</sentences>\n";

	// list of word + pos tag for title words
	sXML << "<title>\n";

	String sTitle = p_Title->s_Text;
	sTitle.Replace ("<", "&lt;");
	sTitle.Replace (">", "&gt;");
	sXML << "<text>" << sTitle << "</text>\n";
	ITERATE_COLLECTION (Word_list_t, p_Title->lst_Words, ite)
	{
		Word* pWord = *ite;
		sXML << "<word text=\"" << pWord->s_Word 
			 << "\" tag=\"" << pWord->s_POSTag 
			 << "\"/>\n";
	}
	
	sXML << "</title>\n";

	// list of words + details for all words in section
	sXML << "<body>\n";
	ITERATE_COLLECTION (Word_dq_t, dq_UniqueWords, iteWord)
	{
		Word* pWord = *iteWord;
		sXML << "<word text=\"" << pWord->s_Word 
			 << "\" tag=\"" << pWord->s_POSTag 
			 << "\" count=\"" << BookSection::GetUnigramOccurrence (pWord->s_Word, false)
			 << "\" fobs=\"" << BookSection::GetFirstOccurrenceBySentence (pWord->s_Stem)
			 << "\" fobw=\"" << BookSection::GetFirstOccurrenceByWord (pWord->s_Stem)
			 << "\"/>\n";
	}
	ITERATE_COLLECTION (Ngram_to_Count_map_t, map_NP_to_Count, iteNP)
	{
		sXML << "<np text=\"" << iteNP->first 
			 << "\" count=\"" << iteNP->second
			 << "\"/>\n";
	}
	sXML << "</body>\n";

	ITERATE_COLLECTION (BookSection_list_t, lst_Children, iteChildren)
	{
		BookSection* pSection = *iteChildren;
		sXML << pSection->GetSectionXML ();
	}

	sXML << "</section>\n";
	return sXML;
}


//-------------------------------------------------------------------
String BookSection::GetXML (void)
{
	String sXML;
	sXML << "<root>\n";
	ITERATE_COLLECTION (Ngram_to_Count_map_t, p_Root->map_NP_to_Corpus_Count, iteNP)
	{
		sXML << "<np text=\"" << iteNP->first 
			 << "\" count=\"" << iteNP->second
			 << "\"/>\n";
	}
	ITERATE_COLLECTION (BookSection_list_t, p_Root->lst_Children, iteChildren)
	{
		BookSection* pSection = *iteChildren;
		sXML << pSection->GetSectionXML ();
	}
	sXML << "</root>\n";

	return sXML;
}


