#ifndef __TITLE_GEN_BOOK_SECTION__
#define __TITLE_GEN_BOOK_SECTION__

#include <list>
#include <deque>
#include <map>
#include <set>
#include "hash_map.hpp"
#include "nlp_string.h"
#include "Word.h"
#include <iostream>
#include <fstream>


using namespace std;


class BookSection;

typedef list <BookSection*>			BookSection_list_t;
typedef deque <BookSection*>		BookSection_dq_t;



//======================================================
#define USE_WORD	'w'
#define USE_STEM	's'

class BookSection
{
	private:
		static Unigram_to_Count_map_t	map_StemToSectionCount;
		static Unigram_to_Count_map_t	map_WordToSectionCount;
		static Unigram_to_Count_map_t	map_Stem_to_First_Occurrence_by_Section;
		Unigram_to_Count_map_t			map_Stem_to_Count;
		Unigram_to_Count_map_t			map_Word_to_Count;
		FrequencyRank_By_Stem_map_t		map_FrequencyRankByStem;
		Unigram_to_Count_map_t			map_Stem_to_First_Occurrence_by_Word;
		Unigram_to_Count_map_t			map_Stem_to_First_Occurrence_by_Sentence;
		Ngram_to_Count_map_t			map_NP_to_Corpus_Count;
		Ngram_to_Count_map_t			map_NP_to_Count;
		
		int 		i_TotalUnigrams;
 		int			i_Depth;
		
		void ParseBody (void);

	public:
		static BookSection_list_t	lst_AllSections;
		static int				i_TotalSections;
		static BookSection*		p_Root;

		BookSection_list_t			lst_Children;
		BookSection*			p_Parent;
		BookSection*			p_PreviousSection;
		BookSection*			p_PreviousSibling;
		BookSection*			p_NextSibling;
		Sentence*				p_Title;
		Sentence_list_t			lst_Sentences;
		Word_dq_t				dq_UniqueWords;

		BookSection (void);
		BookSection (const char* _zTitle, const char* _zBody);
		~BookSection (void);

		void AddChild (BookSection* _pChild);
		BookSection* GetRoot (void);
		void SetText (const char* _zTitle, const char* _zBody);
		void CreateUniqueWordList (void);

		void BuildRootStructures (void);
		void BuildFirstSectionWithStemMap (void);
		void BuildNPCounts (void);
		void BuildUnigramToSectionCounts (void);

		Unigram_list_t CapitalizeTitle (Word_list_t _lstTitleWords);

		bool IsRoot (void);
		BookSection_list_t* GetAllSections (void);
		void GetDescendants (BookSection_list_t* _rlstSections);
		Word_list_t GetBodyWords (void);	

		bool HasUnigram (const char* _zUnigram, bool _bUseStem);
		int GetUnigramOccurrence (const char* _zUnigram, bool _bUseStem);
		int GetNumberOfSectionsWithUnigram (const char* _zUnigram, bool _bUseStem);
		int GetTotalUnigramOccurrences (void);
		long GetFirstOccurrenceByWord (const char* _zStem);
		long GetFirstOccurrenceBySentence (const char* _zStem);
		long GetFirstOccurrenceBySection (const char* _zStem);
		int GetNPCount (const char* _zNP);
		set<String> GetBodyNPs (void);
		int GetCorpusNPCount (const char* _zNP);
		bool PreviousSiblingHasStem (const char* _zStem);
		bool NextSiblingHasStem (const char* _zStem);
		int GetDepth( void );

		static Unigram_to_Count_map_t* GetAllWords (void);

		String GetSectionXML (void);
		static String GetXML (void);
};




#endif
