#ifndef __TITLE_GEN_DECODER__
#define __TITLE_GEN_DECODER__

#include <set>
#include <list>
#include <deque>
#include <iostream>
#include "hash_map.hpp"
#include "Section.h"
#include "FeatureVector.h"
#include "Perceptron.h"
#include "Model.h"

#define TRAIN	true
#define DECODE	false
#define FLAT	true
#define TOC		false


//-----------------------------------------------------------
typedef deque <Word*>					Word_dq_t;
typedef deque <bool>					Flag_dq_t;
typedef list <Word*>					Word_list_t;
typedef hash_map <String, bool>			TitleStem_hash_t;

//-----------------------------------------------------------
class Path
{
	public:
		static int			i_InitialStartWords;
		static Word*		p_StartWord;

		FeatureValue_map_t	hmp_AverageFeatureCounts;
		TitleStem_hash_t	hmp_TitleStems;
		Word_dq_t			dq_Words;
		FeatureVector*		p_FeatureVector;
		double				d_Score;
		double				d_Confidence;
		bool				b_LastWordIsNotInSection;
		bool				b_YGoodMatch;

		double	arr_SectionFeatureTotals [FEATURE_COUNT];
		long	arr_SectionFeatureCounts [FEATURE_COUNT];

		Path (void);
		Path (Path& _rOld);
		~Path (void);
		void Grow (Word* _pWord, double _dScore, 
				   FeatureVector* _pFeatureVector, 
				   bool _bWordIsNotInSection = false);
		FeatureVector* GetFeatureVector();
		void AccumulateFeatureVector (FeatureVector* _pFeatureVector);
		void GetSectionAverageFeatures (FeatureVector& _rFeatureVector);
		void AccumulateSectionFeatures (void);
		int GetSimilarity( Path* pPath );

		bool HasWord (Word& _rWord);
		bool IsYGood(Path& _oYGood);
		bool operator< (Path& _rRight);
};

ostream& operator<< (ostream& _ofs, const Path& _rPath);


//-----------------------------------------------------------
class Path;
class PathStore;
class Beam;
class Decoder;
class Trainer;
typedef multimap <Path*, Path*>			Path_map_t;
typedef multimap <double, PathStore*>	Score_to_PathWord_map_t;



//-----------------------------------------------------------
class PathStore
{
	public:
		Path*			p_Path;
		Word*			p_Word;
		FeatureVector*	p_FeatureVector;
		double			d_Score;

		PathStore (Path* _pPath, Word* _pWord, 
					FeatureVector* _pFeatureVector,
					double _dScore);
		~PathStore (void);
		bool operator< (const PathStore& _rRight) const;
};


//-----------------------------------------------------------
class Beam
{
	private:
		Score_to_PathWord_map_t	map_AllNewPaths;
		Path**					pp_Paths;
		double					d_CurrentMin;
		int						i_MaxBeamWidth;
		
	public:
		int	i_BeamWidth;

		Beam (bool _bTrain);
		Beam ( Beam &_rOld, bool _bCollapse );
		~Beam (void);

		Path* operator[] (int _iIndex);
		void AddPath (Path* _pPath, Word* _pWord, FeatureVector* _pFeatureVector, double _dScore);
		void Trim (void);
		void GetFeatureVectors (Feature_Vector_list_t& _lstFeatureVectors);

		bool HasYGood ( Path& _rYGood );
		void Reset ( Path& _rYGood );
		void Clear ();
		void Init ();
};


//-----------------------------------------------------------
class Decoder
{
	private:
		Model*	p_Model;

	public:
		Beam	beam;
		Path*	p_YGood;

		Decoder (Model& _rModel, bool _bTrain);
		~Decoder (void);

		String BeamSearch (Section* _pSection, bool _bTrain, bool _bFlat);
};

#endif
