#include "FeatureEnumerator.h"
#include <assert.h>


NameToIndex_map_t	FeatureEnumerator::hmp_NameToFeature;
NameToIndex_map_t	FeatureEnumerator::hmp_TocNameToFeature;
String				FeatureEnumerator::arr_FeatureToName [FEATURE_COUNT];
String				FeatureEnumerator::arr_TocFeatureToName [FEATURE_COUNT];


//-------------------------------------------------------------------------
int FeatureEnumerator::NameToIndex (const char* _zName)
{
	NameToIndex_map_t::iterator	ite;
	ite = hmp_NameToFeature.find (_zName);
	if (ite == hmp_NameToFeature.end ())
		return -1;

	return ite->second;
}


//-------------------------------------------------------------------------
int FeatureEnumerator::TocNameToIndex (const char* _zName)
{
	NameToIndex_map_t::iterator	ite;
	ite = hmp_TocNameToFeature.find (_zName);
	if (ite == hmp_TocNameToFeature.end ())
		return -1;

	return ite->second;
}


//-------------------------------------------------------------------------
String FeatureEnumerator::IndexToName (int _iIndex)
{
	assert (_iIndex < FEATURE_COUNT);
	return arr_FeatureToName [_iIndex];
}


//-------------------------------------------------------------------------
String FeatureEnumerator::TocIndexToName (int _iIndex)
{
	assert (_iIndex < FEATURE_COUNT);
	return arr_TocFeatureToName [_iIndex];
}


//-------------------------------------------------------------------------
void FeatureEnumerator::Init (void)
{
	assert (BASE_FEATURE_COUNT > PT_WRB);

	// initialize index to feature name array...

	arr_FeatureToName [ETLM] = "etlm";
	arr_FeatureToName [TLM] = "tlm";
	arr_FeatureToName [L2LM] = "l2lm";
	arr_FeatureToName [LLM] = "llm";
	arr_FeatureToName [ISW] = "isw";
	arr_FeatureToName [IAW] = "iaw";
	arr_FeatureToName [PSHS] = "pshs";
	arr_FeatureToName [NSHS] = "nshs";
	arr_FeatureToName [PHS] = "phs";
	arr_FeatureToName [STFIDF] = "stfidf";
	arr_FeatureToName [ATFIDF] = "atfidf";
	arr_FeatureToName [TFIDF] = "tfidf";
	arr_FeatureToName [FOBSN] = "fobsn";
	arr_FeatureToName [FOBS0] = "fobs0";
	arr_FeatureToName [FOBS1] = "fobs1";
	arr_FeatureToName [TFIDF1] = "tfidf1";
	arr_FeatureToName [FOBS2] = "fobs2";
	arr_FeatureToName [TFIDF2] = "tfidf2";
	arr_FeatureToName [FOBS3] = "fobs3";
	arr_FeatureToName [TFIDF3] = "tfidf3";
	arr_FeatureToName [FOBW] = "fobw";
	arr_FeatureToName [NPR] = "npr";
	arr_FeatureToName [PT_NN] = "pt_nn";
	arr_FeatureToName [PT_LRB] = "pt_lrb";
	arr_FeatureToName [PT_RRB] = "pt_rrb";
	arr_FeatureToName [PT_CC] = "pt_cc";
	arr_FeatureToName [PT_CD] = "pt_cd";
	arr_FeatureToName [PT_DT] = "pt_dt";
	arr_FeatureToName [PT_EX] = "pt_ex";
	arr_FeatureToName [PT_FW] = "pt_fw";
	arr_FeatureToName [PT_IN] = "pt_in";
	arr_FeatureToName [PT_JJ] = "pt_jj";
	arr_FeatureToName [PT_JJR] = "pt_jjr";
	arr_FeatureToName [PT_JJS] = "pt_jjs";
	arr_FeatureToName [PT_LS] = "pt_ls";
	arr_FeatureToName [PT_MD] = "pt_md";
	arr_FeatureToName [PT_NN] = "pt_nn";
	arr_FeatureToName [PT_NNP] = "pt_nnp";
	arr_FeatureToName [PT_NNPS] = "pt_nnps";
	arr_FeatureToName [PT_NNS] = "pt_nns";
	arr_FeatureToName [PT_PDT] = "pt_pdt";
	arr_FeatureToName [PT_POS] = "pt_pos";
	arr_FeatureToName [PT_PRP$] = "pt_prp$";
	arr_FeatureToName [PT_RB] = "pt_rb";
	arr_FeatureToName [PT_RBR] = "pt_rbr";
	arr_FeatureToName [PT_RBS] = "pt_rbs";
	arr_FeatureToName [PT_RP] = "pt_rp";
	arr_FeatureToName [PT_TO] = "pt_to";
	arr_FeatureToName [PT_UH] = "pt_uh";
	arr_FeatureToName [PT_VB] = "pt_vb";
	arr_FeatureToName [PT_VBD] = "pt_vbd";
	arr_FeatureToName [PT_VBG] = "pt_vbg";
	arr_FeatureToName [PT_VBN] = "pt_vbn";
	arr_FeatureToName [PT_VBP] = "pt_vbp";
	arr_FeatureToName [PT_VBZ] = "pt_vbz";
	arr_FeatureToName [PT_WDT] = "pt_wdt";
	arr_FeatureToName [PT_WP] = "pt_wp";
	arr_FeatureToName [PT_WP$] = "pt_wp$";
	arr_FeatureToName [PT_WRB] = "pt_wrb";
	
	arr_FeatureToName [A_ETLM] = "*a_etlm";
	arr_FeatureToName [A_TLM] = "*a_tlm";
	arr_FeatureToName [A_L2LM] = "*a_l2lm";
	arr_FeatureToName [A_LLM] = "*a_llm";
	arr_FeatureToName [A_ISW] = "*a_isw";
	arr_FeatureToName [A_IAW] = "*a_iaw";
	arr_FeatureToName [A_PSHS] = "*a_pshs";
	arr_FeatureToName [A_NSHS] = "*a_nshs";
	arr_FeatureToName [A_PHS] = "*a_phs";
	arr_FeatureToName [A_STFIDF] = "*a_stfidf";
	arr_FeatureToName [A_ATFIDF] = "*a_atfidf";
	arr_FeatureToName [A_TFIDF] = "*a_tfidf";
	arr_FeatureToName [A_FOBSN] = "*a_fobsn";
	arr_FeatureToName [A_FOBS0] = "*a_fobs0";
	arr_FeatureToName [A_FOBS1] = "*a_fobs1";
	arr_FeatureToName [A_TFIDF1] = "*a_tfidf1";
	arr_FeatureToName [A_FOBS2] = "*a_fobs2";
	arr_FeatureToName [A_TFIDF2] = "*a_tfidf2";
	arr_FeatureToName [A_FOBS3] = "*a_fobs3";
	arr_FeatureToName [A_TFIDF3] = "*a_tfidf3";
	arr_FeatureToName [A_FOBW] = "*a_fobw";
	arr_FeatureToName [A_NPR] = "*a_npr";
	arr_FeatureToName [A_PT_NN] = "*a_pt_nn";
	arr_FeatureToName [A_PT_LRB] = "*a_pt_lrb";
	arr_FeatureToName [A_PT_RRB] = "*a_pt_rrb";
	arr_FeatureToName [A_PT_CC] = "*a_pt_cc";
	arr_FeatureToName [A_PT_CD] = "*a_pt_cd";
	arr_FeatureToName [A_PT_DT] = "*a_pt_dt";
	arr_FeatureToName [A_PT_EX] = "*a_pt_ex";
	arr_FeatureToName [A_PT_FW] = "*a_pt_fw";
	arr_FeatureToName [A_PT_IN] = "*a_pt_in";
	arr_FeatureToName [A_PT_JJ] = "*a_pt_jj";
	arr_FeatureToName [A_PT_JJR] = "*a_pt_jjr";
	arr_FeatureToName [A_PT_JJS] = "*a_pt_jjs";
	arr_FeatureToName [A_PT_LS] = "*a_pt_ls";
	arr_FeatureToName [A_PT_MD] = "*a_pt_md";
	arr_FeatureToName [A_PT_NN] = "*a_pt_nn";
	arr_FeatureToName [A_PT_NNP] = "*a_pt_nnp";
	arr_FeatureToName [A_PT_NNPS] = "*a_pt_nnps";
	arr_FeatureToName [A_PT_NNS] = "*a_pt_nns";
	arr_FeatureToName [A_PT_PDT] = "*a_pt_pdt";
	arr_FeatureToName [A_PT_POS] = "*a_pt_pos";
	arr_FeatureToName [A_PT_PRP$] = "*a_pt_prp$";
	arr_FeatureToName [A_PT_RB] = "*a_pt_rb";
	arr_FeatureToName [A_PT_RBR] = "*a_pt_rbr";
	arr_FeatureToName [A_PT_RBS] = "*a_pt_rbs";
	arr_FeatureToName [A_PT_RP] = "*a_pt_rp";
	arr_FeatureToName [A_PT_TO] = "*a_pt_to";
	arr_FeatureToName [A_PT_UH] = "*a_pt_uh";
	arr_FeatureToName [A_PT_VB] = "*a_pt_vb";
	arr_FeatureToName [A_PT_VBD] = "*a_pt_vbd";
	arr_FeatureToName [A_PT_VBG] = "*a_pt_vbg";
	arr_FeatureToName [A_PT_VBN] = "*a_pt_vbn";
	arr_FeatureToName [A_PT_VBP] = "*a_pt_vbp";
	arr_FeatureToName [A_PT_VBZ] = "*a_pt_vbz";
	arr_FeatureToName [A_PT_WDT] = "*a_pt_wdt";
	arr_FeatureToName [A_PT_WP] = "*a_pt_wp";
	arr_FeatureToName [A_PT_WP$] = "*a_pt_wp$";
	arr_FeatureToName [A_PT_WRB] = "*a_pt_wrb";

	for (int i = 0; i < FEATURE_COUNT; ++ i)
		arr_TocFeatureToName [i] = arr_FeatureToName [i];

	arr_TocFeatureToName [H_SPS] = "h_sps";
	arr_TocFeatureToName [H_SPSL] = "h_spsl";
	arr_TocFeatureToName [HP_ST] = "hp_st";
	arr_TocFeatureToName [HP_SS] = "hp_ss";
	arr_TocFeatureToName [HP_SFW] = "hp_sfw";
	arr_TocFeatureToName [HP_SLW] = "hp_slw";
	arr_TocFeatureToName [HP_CS] = "hp_cs";
	arr_TocFeatureToName [HP_TM] = "hp_tm";
	arr_TocFeatureToName [HS_ST] = "hs_st";
	arr_TocFeatureToName [HS_SSM] = "hs_ssm";
	arr_TocFeatureToName [HS_SSA] = "hs_ssa";
	arr_TocFeatureToName [HS_SFW] = "hs_sfw";
	arr_TocFeatureToName [HS_SLW] = "hs_slw";
	arr_TocFeatureToName [HS_CS] = "hs_cs";
	arr_TocFeatureToName [HS_TMA] = "hs_tma";
	arr_TocFeatureToName [H_DT] = "h_dt";
	arr_TocFeatureToName [H_SPR] = "h_spr";
	arr_TocFeatureToName [HPRE_SS] = "hpre_ss";
	arr_TocFeatureToName [HPRE_ST] = "hpre_st";
	arr_TocFeatureToName [HPRE2_ST] = "hpre2_st";
	arr_TocFeatureToName [HC_ST] = "hc_st";
	arr_TocFeatureToName [HR_JJE] = "hr_jje";

	arr_TocFeatureToName [A_H_SPSL] = "*a_h_spsl";
	arr_TocFeatureToName [A_HP_ST] = "*a_hp_st";
	arr_TocFeatureToName [A_HP_SS] = "*a_hp_ss";
	arr_TocFeatureToName [A_HP_SFW] = "*a_hp_sfw";
	arr_TocFeatureToName [A_HP_SLW] = "*a_hp_slw";
	arr_TocFeatureToName [A_HP_CS] = "*a_hp_cs";
	arr_TocFeatureToName [A_HP_TM] = "*a_hp_tm";
	arr_TocFeatureToName [A_HS_ST] = "*a_hs_st";
	arr_TocFeatureToName [A_HS_SSM] = "*a_hs_ssm";
	arr_TocFeatureToName [A_HS_SSA] = "*a_hs_ssa";
	arr_TocFeatureToName [A_HS_SFW] = "*a_hs_sfw";
	arr_TocFeatureToName [A_HS_SLW] = "*a_hs_slw";
	arr_TocFeatureToName [A_HS_CS] = "*a_hs_cs";
	arr_TocFeatureToName [A_HS_TMA] = "*a_hs_tma";
	arr_TocFeatureToName [A_H_DT] = "*a_h_dt";
	arr_TocFeatureToName [A_H_SPR] = "*a_h_spr";
	arr_TocFeatureToName [A_HPRE_SS] = "*a_hpre_ss";
	arr_TocFeatureToName [A_HPRE_ST] = "*a_hpre_st";
	arr_TocFeatureToName [A_HPRE2_ST] = "*a_hpre2_st";
	arr_TocFeatureToName [A_HC_ST] = "*a_hc_st";
	arr_TocFeatureToName [A_HR_JJE] = "*a_hr_jje";

	// initialize feature name to index map...	
	hmp_NameToFeature ["etlm"] = ETLM;
	hmp_NameToFeature ["tlm"] = TLM;
	hmp_NameToFeature ["l2lm"] = L2LM;
	hmp_NameToFeature ["llm"] = LLM;
	hmp_NameToFeature ["isw"] = ISW;
	hmp_NameToFeature ["iaw"] = IAW;
	hmp_NameToFeature ["pshs"] = PSHS;
	hmp_NameToFeature ["nshs"] = NSHS;
	hmp_NameToFeature ["phs"] = PHS;
	hmp_NameToFeature ["stfidf"] = STFIDF;
	hmp_NameToFeature ["atfidf"] = ATFIDF;
	hmp_NameToFeature ["tfidf"] = TFIDF;
	hmp_NameToFeature ["fobsn"] = FOBSN;
	hmp_NameToFeature ["fobs0"] = FOBS0;
	hmp_NameToFeature ["fobs1"] = FOBS1;
	hmp_NameToFeature ["tfidf1"] = TFIDF1;
	hmp_NameToFeature ["fobs2"] = FOBS2;
	hmp_NameToFeature ["tfidf2"] = TFIDF2;
	hmp_NameToFeature ["fobs3"] = FOBS3;
	hmp_NameToFeature ["tfidf3"] = TFIDF3;
	hmp_NameToFeature ["fobw"] = FOBW;
	hmp_NameToFeature ["npr"] = NPR;
	hmp_NameToFeature ["pt_nn"] = PT_NN;
	hmp_NameToFeature ["pt_lrb"] = PT_LRB;
	hmp_NameToFeature ["pt_rrb"] = PT_RRB;
	hmp_NameToFeature ["pt_cc"] = PT_CC;
	hmp_NameToFeature ["pt_cd"] = PT_CD;
	hmp_NameToFeature ["pt_dt"] = PT_DT;
	hmp_NameToFeature ["pt_ex"] = PT_EX;
	hmp_NameToFeature ["pt_fw"] = PT_FW;
	hmp_NameToFeature ["pt_in"] = PT_IN;
	hmp_NameToFeature ["pt_jj"] = PT_JJ;
	hmp_NameToFeature ["pt_jjr"] = PT_JJR;
	hmp_NameToFeature ["pt_jjs"] = PT_JJS;
	hmp_NameToFeature ["pt_ls"] = PT_LS;
	hmp_NameToFeature ["pt_md"] = PT_MD;
	hmp_NameToFeature ["pt_nn"] = PT_NN;
	hmp_NameToFeature ["pt_nnp"] = PT_NNP;
	hmp_NameToFeature ["pt_nnps"] = PT_NNPS;
	hmp_NameToFeature ["pt_nns"] = PT_NNS;
	hmp_NameToFeature ["pt_pdt"] = PT_PDT;
	hmp_NameToFeature ["pt_pos"] = PT_POS;
	hmp_NameToFeature ["pt_prp$"] = PT_PRP$;
	hmp_NameToFeature ["pt_rb"] = PT_RB;
	hmp_NameToFeature ["pt_rbr"] = PT_RBR;
	hmp_NameToFeature ["pt_rbs"] = PT_RBS;
	hmp_NameToFeature ["pt_rp"] = PT_RP;
	hmp_NameToFeature ["pt_to"] = PT_TO;
	hmp_NameToFeature ["pt_uh"] = PT_UH;
	hmp_NameToFeature ["pt_vb"] = PT_VB;
	hmp_NameToFeature ["pt_vbd"] = PT_VBD;
	hmp_NameToFeature ["pt_vbg"] = PT_VBG;
	hmp_NameToFeature ["pt_vbn"] = PT_VBN;
	hmp_NameToFeature ["pt_vbp"] = PT_VBP;
	hmp_NameToFeature ["pt_vbz"] = PT_VBZ;
	hmp_NameToFeature ["pt_wdt"] = PT_WDT;
	hmp_NameToFeature ["pt_wp"] = PT_WP;
	hmp_NameToFeature ["pt_wp$"] = PT_WP$;
	hmp_NameToFeature ["pt_wrb"] = PT_WRB;

	hmp_NameToFeature ["*a_etlm"] = A_ETLM;
	hmp_NameToFeature ["*a_tlm"] = A_TLM;
	hmp_NameToFeature ["*a_l2lm"] = A_L2LM;
	hmp_NameToFeature ["*a_llm"] = A_LLM;
	hmp_NameToFeature ["*a_isw"] = A_ISW;
	hmp_NameToFeature ["*a_iaw"] = A_IAW;
	hmp_NameToFeature ["*a_pshs"] = A_PSHS;
	hmp_NameToFeature ["*a_nshs"] = A_NSHS;
	hmp_NameToFeature ["*a_phs"] = A_PHS;
	hmp_NameToFeature ["*a_stfidf"] = A_STFIDF;
	hmp_NameToFeature ["*a_atfidf"] = A_ATFIDF;
	hmp_NameToFeature ["*a_tfidf"] = A_TFIDF;
	hmp_NameToFeature ["*a_fobsn"] = A_FOBSN;
	hmp_NameToFeature ["*a_fobs0"] = A_FOBS0;
	hmp_NameToFeature ["*a_fobs1"] = A_FOBS1;
	hmp_NameToFeature ["*a_tfidf1"] = A_TFIDF1;
	hmp_NameToFeature ["*a_fobs2"] = A_FOBS2;
	hmp_NameToFeature ["*a_tfidf2"] = A_TFIDF2;
	hmp_NameToFeature ["*a_fobs3"] = A_FOBS3;
	hmp_NameToFeature ["*a_tfidf3"] = A_TFIDF3;
	hmp_NameToFeature ["*a_fobw"] = A_FOBW;
	hmp_NameToFeature ["*a_npr"] = A_NPR;
	hmp_NameToFeature ["*a_pt_nn"] = A_PT_NN;
	hmp_NameToFeature ["*a_pt_lrb"] = A_PT_LRB;
	hmp_NameToFeature ["*a_pt_rrb"] = A_PT_RRB;
	hmp_NameToFeature ["*a_pt_cc"] = A_PT_CC;
	hmp_NameToFeature ["*a_pt_cd"] = A_PT_CD;
	hmp_NameToFeature ["*a_pt_dt"] = A_PT_DT;
	hmp_NameToFeature ["*a_pt_ex"] = A_PT_EX;
	hmp_NameToFeature ["*a_pt_fw"] = A_PT_FW;
	hmp_NameToFeature ["*a_pt_in"] = A_PT_IN;
	hmp_NameToFeature ["*a_pt_jj"] = A_PT_JJ;
	hmp_NameToFeature ["*a_pt_jjr"] = A_PT_JJR;
	hmp_NameToFeature ["*a_pt_jjs"] = A_PT_JJS;
	hmp_NameToFeature ["*a_pt_ls"] = A_PT_LS;
	hmp_NameToFeature ["*a_pt_md"] = A_PT_MD;
	hmp_NameToFeature ["*a_pt_nn"] = A_PT_NN;
	hmp_NameToFeature ["*a_pt_nnp"] = A_PT_NNP;
	hmp_NameToFeature ["*a_pt_nnps"] = A_PT_NNPS;
	hmp_NameToFeature ["*a_pt_nns"] = A_PT_NNS;
	hmp_NameToFeature ["*a_pt_pdt"] = A_PT_PDT;
	hmp_NameToFeature ["*a_pt_pos"] = A_PT_POS;
	hmp_NameToFeature ["*a_pt_prp$"] = A_PT_PRP$;
	hmp_NameToFeature ["*a_pt_rb"] = A_PT_RB;
	hmp_NameToFeature ["*a_pt_rbr"] = A_PT_RBR;
	hmp_NameToFeature ["*a_pt_rbs"] = A_PT_RBS;
	hmp_NameToFeature ["*a_pt_rp"] = A_PT_RP;
	hmp_NameToFeature ["*a_pt_to"] = A_PT_TO;
	hmp_NameToFeature ["*a_pt_uh"] = A_PT_UH;
	hmp_NameToFeature ["*a_pt_vb"] = A_PT_VB;
	hmp_NameToFeature ["*a_pt_vbd"] = A_PT_VBD;
	hmp_NameToFeature ["*a_pt_vbg"] = A_PT_VBG;
	hmp_NameToFeature ["*a_pt_vbn"] = A_PT_VBN;
	hmp_NameToFeature ["*a_pt_vbp"] = A_PT_VBP;
	hmp_NameToFeature ["*a_pt_vbz"] = A_PT_VBZ;
	hmp_NameToFeature ["*a_pt_wdt"] = A_PT_WDT;
	hmp_NameToFeature ["*a_pt_wp"] = A_PT_WP;
	hmp_NameToFeature ["*a_pt_wp$"] = A_PT_WP$;
	hmp_NameToFeature ["*a_pt_wrb"] = A_PT_WRB;
	

	hmp_TocNameToFeature = hmp_NameToFeature;
	
	hmp_TocNameToFeature ["h_sps"] = H_SPS;
	hmp_TocNameToFeature ["h_spsl"] = H_SPSL;
	hmp_TocNameToFeature ["hp_st"] = HP_ST;
	hmp_TocNameToFeature ["hp_ss"] = HP_SS;
	hmp_TocNameToFeature ["hp_sfw"] = HP_SFW;
	hmp_TocNameToFeature ["hp_slw"] = HP_SLW;
	hmp_TocNameToFeature ["hp_cs"] = HP_CS;
	hmp_TocNameToFeature ["hp_tm"] = HP_TM;
	hmp_TocNameToFeature ["hs_st"] = HS_ST;
	hmp_TocNameToFeature ["hs_ssm"] = HS_SSM;
	hmp_TocNameToFeature ["hs_ssa"] = HS_SSA;
	hmp_TocNameToFeature ["hs_sfw"] = HS_SFW;
	hmp_TocNameToFeature ["hs_slw"] = HS_SLW;
	hmp_TocNameToFeature ["hs_cs"] = HS_CS;
	hmp_TocNameToFeature ["hs_tma"] = HS_TMA;
	hmp_TocNameToFeature ["h_dt"] = H_DT;
	hmp_TocNameToFeature ["h_spr"] = H_SPR;
	hmp_TocNameToFeature ["hpre_ss"] = HPRE_SS;
	hmp_TocNameToFeature ["hpre_st"] = HPRE_ST;
	hmp_TocNameToFeature ["hpre2_st"] = HPRE2_ST;
	hmp_TocNameToFeature ["hc_st"] = HC_ST;
	hmp_TocNameToFeature ["hr_jje"] = HR_JJE;

	hmp_TocNameToFeature ["*a_h_sps"] = A_H_SPS;
	hmp_TocNameToFeature ["*a_h_spsl"] = A_H_SPSL;
	hmp_TocNameToFeature ["*a_hp_st"] = A_HP_ST;
	hmp_TocNameToFeature ["*a_hp_ss"] = A_HP_SS;
	hmp_TocNameToFeature ["*a_hp_sfw"] = A_HP_SFW;
	hmp_TocNameToFeature ["*a_hp_slw"] = A_HP_SLW;
	hmp_TocNameToFeature ["*a_hp_cs"] = A_HP_CS;
	hmp_TocNameToFeature ["*a_hp_tm"] = A_HP_TM;
	hmp_TocNameToFeature ["*a_hs_st"] = A_HS_ST;
	hmp_TocNameToFeature ["*a_hs_ssm"] = A_HS_SSM;
	hmp_TocNameToFeature ["*a_hs_ssa"] = A_HS_SSA;
	hmp_TocNameToFeature ["*a_hs_sfw"] = A_HS_SFW;
	hmp_TocNameToFeature ["*a_hs_slw"] = A_HS_SLW;
	hmp_TocNameToFeature ["*a_hs_cs"] = A_HS_CS;
	hmp_TocNameToFeature ["*a_hs_tma"] = A_HS_TMA;
	hmp_TocNameToFeature ["*a_h_dt"] = A_H_DT;
	hmp_TocNameToFeature ["*a_h_spr"] = A_H_SPR;
	hmp_TocNameToFeature ["*a_hpre_ss"] = A_HPRE_SS;
	hmp_TocNameToFeature ["*a_hpre_st"] = A_HPRE_ST;
	hmp_TocNameToFeature ["*a_hpre2_st"] = A_HPRE2_ST;
	hmp_TocNameToFeature ["*a_hc_st"] = A_HC_ST;
	hmp_TocNameToFeature ["*a_hr_jje"] = A_HR_JJE;
}

