#include "FeatureEnumerator.h"
#include "FeatureVector.h"
#include "math.h"
#include "Utils.h"
#include "Section.h"
#include "Config.h"
#include "TreeDecoder.h"
#include <math.h>
#include "Decoder.h"

using namespace std;

LanguageModel*	FeatureVector::p_TagLM = NULL;
LanguageModel*	FeatureVector::p_LexicalLM = NULL;
Section_list_t*	FeatureVector::plst_AllSections = NULL;
Section*		FeatureVector::p_RootSection = NULL;
Section*		FeatureVector::p_Section = NULL;
Word*			FeatureVector::p_EndWord = NULL;
Word*			FeatureVector::p_StartWord = NULL;
int				FeatureVector::i_SectionCount = 0;
int				FeatureVector::i_WindowSize = 0;
bool			FeatureVector::b_UseWordAndTagLM = false;



//-------------------------------------------------------------------
FeatureVector::FeatureVector (void)
{
	memset (arr_Features, 0, FEATURE_COUNT * sizeof (double));
}


//-------------------------------------------------------------------
FeatureVector::FeatureVector (FeatureVector& _rOld)
{
	for (int i = 0; i < FEATURE_COUNT; ++ i)
		arr_Features [i] = _rOld.arr_Features [i];
}


//-------------------------------------------------------------------
FeatureVector::~FeatureVector (void)
{
	Clear ();
}


//-------------------------------------------------------------------
void FeatureVector::Init (Model& _rModel, Section* _pSection)
{
	p_TagLM = &_rModel.lm_Tag;
	p_LexicalLM = &_rModel.lm_Lexical;
	i_WindowSize = Config::Get ("WINDOW_SIZE");

	p_RootSection = Section::p_Root;
	p_Section = _pSection;
	p_EndWord = Word::GetEndWord ();
	p_StartWord = Word::GetStartWord ();
	plst_AllSections = p_RootSection->GetAllSections ();
	i_SectionCount = plst_AllSections->size ();
	b_UseWordAndTagLM = (1 == (int)Config::Get ("LM_USE_TAGS"));
}


//-------------------------------------------------------------------
void FeatureVector::Clear (void)
{
	memset (arr_Features, 0, FEATURE_COUNT * sizeof (double));
}

//-------------------------------------------------------------------
void FeatureVector::CreateFeatures (Path* _pPath, Word* _pWord)
{

        Word_dq_t& _dqFullHistory = _pPath->dq_Words;

	if (true == _pWord->b_EndWord) 
	{
		// end tag lm score...
		pair <double, double> pairTagLM = p_TagLM->Get (_dqFullHistory, _pWord, Word::GetTagText, false);
		AddFeature (ETLM, - pairTagLM.first);

		return;  
	}

	// -----------------------------------------------------------------
	// ORDERING FEATURES 												
	// -----------------------------------------------------------------
	
	unsigned int iFullHistorySize = _dqFullHistory.size ();

	// Feature: Title Tag LM  ------------------------------------------
	{
		pair <double, double> pairTagLM = p_TagLM->Get (_dqFullHistory, _pWord, Word::GetTagText, false);
		if (pairTagLM.first == 0)
			cout << "TAG LM = 0 !!" << endl;
		AddFeature (TLM, - pairTagLM.first);
	}

	// Feature: Lexical LM  --------------------------------------------
	if (false == _pWord->b_StartWord)
	{
	    WordRenderer_t wordRenderer = FeatureVector::b_UseWordAndTagLM ? Word::GetWordTagText : Word::GetWordText;
		long lNGramLength = iFullHistorySize - p_LexicalLM->i_Order;
		if (0 == lNGramLength)
		{
			pair <double, double> pairLexicalLM_Out = p_LexicalLM->Get (_dqFullHistory, _pWord, 
																		wordRenderer, true);
			if (pairLexicalLM_Out.first <= -100)
			{
				cout << "   LLM : " << pairLexicalLM_Out.first << " --------------------------" << endl;
				cout << _dqFullHistory << endl;
			}
			AddFeature (L2LM, - pairLexicalLM_Out.first);
		}
		else if (lNGramLength > 0)
		{
			pair <double, double> pairLexicalLM_Out = p_LexicalLM->GetFullScore (_dqFullHistory, _pWord, 
											     wordRenderer, true);
			if (pairLexicalLM_Out.first <= -100)
			{
				cout << "   ELLM : " << pairLexicalLM_Out.first << " --------------------------" << endl;
				cout << _dqFullHistory << endl;
			}
			AddFeature (LLM, - pairLexicalLM_Out.first);
		}
	}


	// -----------------------------------------------------------------
	// Do not calculate the featues below for the end word				
	// SELECTION FEATURES												
	// -----------------------------------------------------------------

	// Feature: POS Tag ------------------------------------------------
	AddPosTagFeature (_pWord->s_POSTag, 1);


	// Feature: Is word a stopword  ------------------------------------
	bool bIsStopWord = _pWord->b_IsStopWord;
	bool bIsAuxiliaryWord = _pWord->b_IsAuxiliaryWord;
	if (true == bIsStopWord)
		AddFeature (ISW, 1.0);
	if (true == bIsAuxiliaryWord)
		AddFeature (IAW, 1.0);

	// Feature: previous/next sibling, parent section has stem  --------
	if (p_Section->PreviousSiblingHasStem (_pWord->s_Stem)) 
		AddFeature (PSHS, 1.0);
	if (p_Section->NextSiblingHasStem (_pWord->s_Stem))
		AddFeature (NSHS, 1.0);
	if ((p_Section->p_Parent != NULL) && (p_Section->p_Parent->HasUnigram (_pWord->s_Stem, true)))
		AddFeature (PHS, 1.0);


	// Feature: TF IDF  ------------------------------------------------
	double dTotalUnigramOccurrences = p_Section->GetTotalUnigramOccurrences ();

	double dTF = p_Section->GetUnigramOccurrence (_pWord->s_Stem, true) / dTotalUnigramOccurrences;
	double dDF = p_RootSection->GetNumberOfSectionsWithUnigram (_pWord->s_Stem, true) / (double) i_SectionCount;
	double dTFIDF = - dTF / log (dDF);
	if (true == bIsStopWord) 
		AddFeature (STFIDF, dTFIDF);
	else if (true == bIsAuxiliaryWord)
		AddFeature (ATFIDF, dTFIDF); 
	else
		AddFeature (TFIDF, dTFIDF); 

	// Feature: first occurrence in section by sentence, without normalization
	long lFirstOccurrence = p_Section->GetFirstOccurrenceBySentence (_pWord->s_Stem);
	double dFirstOccurrenceNormalized = lFirstOccurrence / (double) p_Section->GetSentenceCount ();
	if (true == _pWord->b_IsContentWord)
	{
		AddFeature (FOBSN, dFirstOccurrenceNormalized);
		if (lFirstOccurrence == 0)
			AddFeature (FOBS0, 1);
		if (dFirstOccurrenceNormalized < 1/ (double)3)
		{
			AddFeature (FOBS1, 1);
			AddFeature (TFIDF1, dTFIDF); 
		}
		else if (dFirstOccurrenceNormalized < 2/ (double)3)
		{
			AddFeature (FOBS2, 1);
			AddFeature (TFIDF2, dTFIDF); 
		}
		else
		{
			AddFeature (FOBS3, 1);
			AddFeature (TFIDF3, dTFIDF); 
		}
	}

	// Feature: first occurrence in section by word  -------------------
	{
		// TODO: Add caching?
		double dFirstOccByWordNormalized = p_Section->GetFirstOccurrenceByWord (_pWord->s_Stem) 
											/ dTotalUnigramOccurrences;
		AddFeature (FOBW, dFirstOccByWordNormalized);
	}

	// Feature: NP count in section  -----------------------------------
	if ((iFullHistorySize > p_LexicalLM->i_Order) && (Utils::set_NP_Tags.find (_pWord->s_POSTag) != Utils::set_NP_Tags.end ()))
	{
		Word_dq_t	dqAllWords (_dqFullHistory);
		dqAllWords.push_back (_pWord);
		
		Ngram_list_t lstNPs;
		Utils::ExtractNounPhrases (lstNPs, dqAllWords);

		if ((lstNPs.size () > 0) && (lstNPs.back ()->back ()->s_Word == _pWord->s_Word)) 
		{
			String sLastNP;
			Word_list_t* pNPs = lstNPs.back ();
			Utils::WordListToNgramString (sLastNP, pNPs);

			long lSectionNPCount = p_Section->GetNPCount (sLastNP);
			long lCorpusNPCount = p_RootSection->GetCorpusNPCount (sLastNP);
			if (lCorpusNPCount > 0) 
				AddFeature (NPR, (lSectionNPCount + 1) / (double) lCorpusNPCount);
		}

		Utils::DeleteNgramList (lstNPs);
	}
}

//-------------------------------------------------------------------
void FeatureVector::AddPosTagFeature (String& _sTag, double dValue)
{
	if (_sTag == "LRB")
		AddFeature (PT_LRB, 1);
	else if (_sTag == "RRB")
		AddFeature (PT_RRB, 1);
	else if (_sTag == "CC")
		AddFeature (PT_CC, 1);
	else if (_sTag == "CD")
		AddFeature (PT_CD, 1);
	else if (_sTag == "DT")
		AddFeature (PT_DT, 1);
	else if (_sTag == "EX")
		AddFeature (PT_EX, 1);
	else if (_sTag == "FW")
		AddFeature (PT_FW, 1);
	else if (_sTag == "IN")
		AddFeature (PT_IN, 1);
	else if (_sTag == "JJ")
		AddFeature (PT_JJ, 1);
	else if (_sTag == "JJR")
		AddFeature (PT_JJR, 1);
	else if (_sTag == "JJS")
		AddFeature (PT_JJS, 1);
	else if (_sTag == "LS")
		AddFeature (PT_LS, 1);
	else if (_sTag == "MD")
		AddFeature (PT_MD, 1);
	else if (_sTag == "NN")
		AddFeature (PT_NN, 1);
	else if (_sTag == "NNP")
		AddFeature (PT_NNP, 1);
	else if (_sTag == "NNPS")
		AddFeature (PT_NNPS, 1);
	else if (_sTag == "NNS")
		AddFeature (PT_NNS, 1);
	else if (_sTag == "PDT")
		AddFeature (PT_PDT, 1);
	else if (_sTag == "POS")
		AddFeature (PT_POS, 1);
	else if (_sTag == "PRP$")
		AddFeature (PT_PRP$, 1);
	else if (_sTag == "RB")
		AddFeature (PT_RB, 1);
	else if (_sTag == "RBR")
		AddFeature (PT_RBR, 1);
	else if (_sTag == "RBS")
		AddFeature (PT_RBS, 1);
	else if (_sTag == "RP")
		AddFeature (PT_RP, 1);
	else if (_sTag == "TO")
		AddFeature (PT_TO, 1);
	else if (_sTag == "UH")
		AddFeature (PT_UH, 1);
	else if (_sTag == "VB")
		AddFeature (PT_VB, 1);
	else if (_sTag == "VBD")
		AddFeature (PT_VBD, 1);
	else if (_sTag == "VBG")
		AddFeature (PT_VBG, 1);
	else if (_sTag == "VBN")
		AddFeature (PT_VBN, 1);
	else if (_sTag == "VBP")
		AddFeature (PT_VBP, 1);
	else if (_sTag == "VBZ")
		AddFeature (PT_VBZ, 1);
	else if (_sTag == "WDT")
		AddFeature (PT_WDT, 1);
	else if (_sTag == "WP")
		AddFeature (PT_WP, 1);
	else if (_sTag == "WP$")
		AddFeature (PT_WP$, 1);
	else if (_sTag == "WRB")
		AddFeature (PT_WRB, 1);
}

//-------------------------------------------------------------------
void FeatureVector::Save (const char* _zFileName, const char* _zTag, bool _bHierarchical)
{
    	ofstream file (_zFileName, ios::app | ios::ate);

	file << "[" << _zTag 
		 << "   " << Config::Get ("TYPE")
		 << ".b" << Config::Get ("BEAM_SIZE") 
		 << "-d" << Config::Get ("DEPTH")
		 << "-i" << Config::Get ("ITERATIONS")
		 << "-l" << Config::Get ("TITLE_LENGTH")
	     << "]" << endl;

	file.precision (15);
	if (true == _bHierarchical)
	{
		for (int i = 0; i < FEATURE_COUNT; ++ i)
			file << FeatureEnumerator::TocIndexToName (i) << ':' << arr_Features [i] << ' ';
	}
	else
	{
		for (int i = 0; i < FEATURE_COUNT; ++ i)
			file << FeatureEnumerator::IndexToName (i) << ':' << arr_Features [i] << ' ';
	}

	file << endl;

	file.close ();
}
