#include "Config.h"
#include "Section.h"
#include "Tagger.h"
#include "Reader.h"
#include "Decoder.h"
#include "TreeDecoder.h"
#include "LanguageModel.h"
#include "Utils.h"
#include "FeatureEnumerator.h"
#include "nlp_string.h"
#include <sys/time.h>
#include <time.h>


void PrintTableOfContents (Section_dq_t dqTrainingSections, 
							Section_to_Title_map_t mapSectionToTitle, 
							bool _bTOC);



// -------------------------------------------------------------
void TrainFlat (Model& _rModel, Section* _pRoot, Section_list_t& _lstSections)
{
	// load perceptron weights if necessary...					
	String sFileName (Config::Get ("WEIGHTS_FILE_PATH"));

	if (1 != (int) Config::Get ("RESET_WEIGHTS"))
	{
		cout << "* Loading perceptron weights" << endl;
		_rModel.o_TitlePerceptron.Load (sFileName);
	}
	else
		cout << "* Resetting perceptron weights" << endl;

	// train...													
	cout << "* Training" << endl;
	Decoder trainer (_rModel, TRAIN);
	TreeDecoder::b_HierarchicalDecoding = false;
	FeatureVector::b_UseHierarchicalFeatures = false;

	int iEndSection = _lstSections.size ();
	int iTotalTrainIters = Config::Get ("TRAIN_ITERATIONS");
	for (int iTrainIters = 0; iTrainIters < iTotalTrainIters; ++iTrainIters) 
	{
		int iSection = 0;
		ITERATE_COLLECTION (Section_list_t, _lstSections, iteSection) 
		{
			Section* pSection = *iteSection;
			cout << "   [" << (iTrainIters + 1) << '/' << iTotalTrainIters 
				 << "][" << iSection << '/' << iEndSection 
				 << "] Training : " << pSection->p_Title->s_Text << " >> " << endl;
			trainer.BeamSearch (pSection, TRAIN, FLAT);
			cout << endl;

			++ iSection;
		}

		// save perceptron weights...
		String sTag;
		sTag << '[' << (iTrainIters + 1) << '/' << iTotalTrainIters << ']';
		// << "][" << i << '/' << iEndSection << ']';
		_rModel.o_TitlePerceptron.Save (sFileName, sTag);
	}

	return;
}



// -------------------------------------------------------------
void DecodeFlat (Model& _rModel, Section* _pRoot, Section_list_t& _lstSections)
{
	cout << "* Loading perceptron weights" << endl;
	String sFileName (Config::Get ("WEIGHTS_FILE_PATH"));
	_rModel.o_TitlePerceptron.LoadReadOnly (sFileName);

	cout << "* Decoding" << endl;
	Decoder decoder (_rModel, DECODE);
	/// TreeDecoder::b_HierarchicalDecoding = false;
	FeatureVector::b_UseHierarchicalFeatures = false;


	cout << "  sections to decode : " << _lstSections.size () << endl;
	//------------------------------------------
	Section_to_Title_map_t mapSectionToTitle;
	
	Section_dq_t dqSections;
	int iEndSection = _lstSections.size ();
	int iSection = 0;
	ITERATE_COLLECTION (Section_list_t, _lstSections, iteSection) 
	{
		Section* pSection = *iteSection;
		cout << "   [" << iSection << '/' << iEndSection 
			 << "] Decoding : " << pSection->p_Title->s_Text << " >> " << endl;

		String sTitle = decoder.BeamSearch (pSection, DECODE, FLAT);
		mapSectionToTitle.insert (make_pair ((long) pSection, sTitle));

		cout << endl;
		dqSections.push_back (pSection);

		++ iSection;
	}

	PrintTableOfContents (dqSections, mapSectionToTitle, false);
	return;
}
