#include "Config.h"
#include "BookSection.h"
#include "Tagger.h"
#include "Reader.h"
#include "Decoder.h"
#include "TreeDecoder.h"
#include "LanguageModel.h"
#include "Utils.h"
#include "FeatureEnumerator.h"
#include "nlp_string.h"
#include <sys/time.h>
#include <time.h>
#include <stdlib.h>



//--------------------------------------------------------------------------------
void GetAllSubtrees (Section* _pRoot, Section_dq_t& _rdqSections)
{
	int iDepth = Config::Get ("TOC_SECTION_DEPTH");
	Utils::SectionsAtDepth (_rdqSections, _pRoot, iDepth);
}


//--------------------------------------------------------------------------------
void RandomizeCorpus (Section* _pRoot)
{
	String sRandomizationFileName;
	sRandomizationFileName = Config::Get ("DATASET_RANDOMIZATION_PATH") + ".rnd";

	for (int i = 10; i >= 0; -- i)
	{
		String sBackupFile (sRandomizationFileName);
		sBackupFile << '.' << i;
		if (0 != access (sBackupFile, F_OK))
			continue;

		if (0 != access (sBackupFile, R_OK))
		{
			cerr << "Don't have read permission on corpus randomization "
					"backup file '" << sBackupFile << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}
		if (0 != access (sBackupFile, W_OK))
		{
			cerr << "Don't have write permission on corpus randomization "
					"backup file '" << sBackupFile << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}

		String sNewBackupFile (sRandomizationFileName);
		sNewBackupFile << '.' << (i + 1);
		if (0 != rename (sBackupFile, sNewBackupFile))
		{
			cerr << "Renaming of backup file '" << sBackupFile << "' to '"
				 << sNewBackupFile << "' failed. Please check file permissions."
				 << endl;
			return;
		}
	}

	if (0 == access (sRandomizationFileName, F_OK))
	{
		if (0 != access (sRandomizationFileName, R_OK))
		{
			cerr << "Don't have read permission on corpus randomization "
					"backup file '" << sRandomizationFileName << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}
		if (0 != access (sRandomizationFileName, W_OK))
		{
			cerr << "Don't have write permission on corpus randomization "
					"backup file '" << sRandomizationFileName << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}

		String sBackupFile (sRandomizationFileName);
		sBackupFile << ".0";
		if (0 != rename (sRandomizationFileName, sBackupFile))
		{
			cerr << "Renaming of backup file '" << sRandomizationFileName << "' to '"
				 << sBackupFile << "' failed. Please check file permissions."
				 << endl;
			return;
		}
	}

	ofstream file (sRandomizationFileName);
	if (false == file.is_open ())
	{
		cerr << "Failed to open corpus randomization file '"
			 << sRandomizationFileName << "' for writing." << endl;
		cerr << "Cannot continue. QUitting." << endl;
		return;
	}

	time_t now = time (NULL);
	file << ctime (&now);

	Section_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);
	int iSubtreeCount = dqAllSubtrees.size ();

	set <int> setAllIndices;
	for (int i = 0; i < iSubtreeCount; ++ i)
		setAllIndices.insert (i);

	srand (time (NULL));
	while (false == setAllIndices.empty ())
	{
		int iIndex = (int) (iSubtreeCount * (rand () / (double) RAND_MAX));
		while (setAllIndices.find (iIndex) == setAllIndices.end ())
			iIndex = (int) (iSubtreeCount * (rand () / (double) RAND_MAX));
		
		setAllIndices.erase (iIndex);
		file << iIndex << ' ';
	}

	file.flush ();
	file.close ();
}


//--------------------------------------------------------------------------------
void GetRandomizedIndices (Section* _pRoot, deque <int>& _dqIndices)
{
	Section_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);
	int iSubtreeCount = dqAllSubtrees.size ();


	String sRandomizationFileName;
	sRandomizationFileName = Config::Get ("DATASET_RANDOMIZATION_PATH") + ".rnd";
	ifstream file (sRandomizationFileName);
	if (false == file.is_open ())
	{
		cerr << "Failed to open corpus randomization file '"
			 << sRandomizationFileName << "' for writing." << endl;
		cerr << "Cannot continue. QUitting." << endl;
		return;
	}

	String sDate;
	getline (file, sDate);
	cout << "  - loading corpus randomization from date " << sDate << endl;

	String sLine;
	getline (file, sLine);

	Unigram_list_t lstIndices;
	sLine.Split (lstIndices);

	ITERATE_COLLECTION (Unigram_list_t, lstIndices, ite)
	{
		int iIndex = *ite;
		_dqIndices.push_back (iIndex);
	}

	int iIndices = _dqIndices.size ();
	if (iSubtreeCount != iIndices)
	{
		cerr << "ERROR : Number of subtrees in corpus (" << iSubtreeCount
			 << ") does not match number of randomized indices (" << iIndices 
			 << ") !  Quitting." << endl;
		return;
	}
}


//--------------------------------------------------------------------------------
void GetTrainingSections (Section* _pRoot, Section_list_t& _lstSections)
{
	deque <int> dqIndices;
	GetRandomizedIndices (_pRoot, dqIndices);
	int iIndices = dqIndices.size ();

	Section_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);

	double dStartRatio = Config::Get ("TOC_TRAIN_START_RATIO");
	double dEndRatio = Config::Get ("TOC_TRAIN_END_RATIO");
	int iStartSection = (int) (dStartRatio * dqIndices.size ());
	int iEndSection = (int) (dEndRatio * dqIndices.size ());

	cout << "  - Training sections : ";
	for (int i = 0; i < iIndices; ++ i)
	{
		if (i >= iEndSection)
			break;
		if (i < iStartSection)
			continue;

		cout << dqIndices [i] << ", ";
		_lstSections.push_back (dqAllSubtrees [dqIndices [i]]);
	}
	cout << endl;
}



//--------------------------------------------------------------------------------
void GetDecodingSections (Section* _pRoot, Section_list_t& _lstSections)
{
	deque <int> dqIndices;
	GetRandomizedIndices (_pRoot, dqIndices);
	int iIndices = dqIndices.size ();

	Section_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);

	double dStartRatio = Config::Get ("TOC_DECODE_START_RATIO");
	double dEndRatio = Config::Get ("TOC_DECODE_END_RATIO");
	int iStartSection = (int) (dStartRatio * dqIndices.size ());
	int iEndSection = (int) (dEndRatio * dqIndices.size ());

	cout << "  - Decoding sections : ";
	for (int i = 0; i < iIndices; ++ i)
	{
		if (i >= iEndSection)
			break;
		if (i < iStartSection)
			continue;

		cout << dqIndices [i] << ", ";
		_lstSections.push_back (dqAllSubtrees [dqIndices [i]]);
	}
	cout << endl;
}







//--------------------------------------------------------------------------------
void GetAllSubtrees (BookSection* _pRoot, BookSection_dq_t& _rdqSections)
{
	int iDepth = Config::Get ("TOC_SECTION_DEPTH");
	Utils::SectionsAtDepth (_rdqSections, _pRoot, iDepth);
}


//--------------------------------------------------------------------------------
void RandomizeCorpus (BookSection* _pRoot)
{
	String sRandomizationFileName;
	sRandomizationFileName = Config::Get ("DATASET_RANDOMIZATION_PATH") + ".rnd";

	for (int i = 10; i >= 0; -- i)
	{
		String sBackupFile (sRandomizationFileName);
		sBackupFile << '.' << i;
		if (0 != access (sBackupFile, F_OK))
			continue;

		if (0 != access (sBackupFile, R_OK))
		{
			cerr << "Don't have read permission on corpus randomization "
					"backup file '" << sBackupFile << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}
		if (0 != access (sBackupFile, W_OK))
		{
			cerr << "Don't have write permission on corpus randomization "
					"backup file '" << sBackupFile << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}

		String sNewBackupFile (sRandomizationFileName);
		sNewBackupFile << '.' << (i + 1);
		if (0 != rename (sBackupFile, sNewBackupFile))
		{
			cerr << "Renaming of backup file '" << sBackupFile << "' to '"
				 << sNewBackupFile << "' failed. Please check file permissions."
				 << endl;
			return;
		}
	}

	if (0 == access (sRandomizationFileName, F_OK))
	{
		if (0 != access (sRandomizationFileName, R_OK))
		{
			cerr << "Don't have read permission on corpus randomization "
					"backup file '" << sRandomizationFileName << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}
		if (0 != access (sRandomizationFileName, W_OK))
		{
			cerr << "Don't have write permission on corpus randomization "
					"backup file '" << sRandomizationFileName << "'." << endl;
			cerr << "Please check file permissions and re-run." << endl;
			return;
		}

		String sBackupFile (sRandomizationFileName);
		sBackupFile << ".0";
		if (0 != rename (sRandomizationFileName, sBackupFile))
		{
			cerr << "Renaming of backup file '" << sRandomizationFileName << "' to '"
				 << sBackupFile << "' failed. Please check file permissions."
				 << endl;
			return;
		}
	}

	ofstream file (sRandomizationFileName);
	if (false == file.is_open ())
	{
		cerr << "Failed to open corpus randomization file '"
			 << sRandomizationFileName << "' for writing." << endl;
		cerr << "Cannot continue. QUitting." << endl;
		return;
	}

	time_t now = time (NULL);
	file << ctime (&now);

	BookSection_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);
	int iSubtreeCount = dqAllSubtrees.size ();

	set <int> setAllIndices;
	for (int i = 0; i < iSubtreeCount; ++ i)
		setAllIndices.insert (i);

	srand (time (NULL));
	while (false == setAllIndices.empty ())
	{
		int iIndex = (int) (iSubtreeCount * (rand () / (double) RAND_MAX));
		while (setAllIndices.find (iIndex) == setAllIndices.end ())
			iIndex = (int) (iSubtreeCount * (rand () / (double) RAND_MAX));
		
		setAllIndices.erase (iIndex);
		file << iIndex << ' ';
	}

	file.flush ();
	file.close ();
}


//--------------------------------------------------------------------------------
void GetRandomizedIndices (BookSection* _pRoot, deque <int>& _dqIndices)
{
	BookSection_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);
	int iSubtreeCount = dqAllSubtrees.size ();


	String sRandomizationFileName;
	sRandomizationFileName = Config::Get ("DATASET_RANDOMIZATION_PATH") + ".rnd";
	ifstream file (sRandomizationFileName);
	if (false == file.is_open ())
	{
		cerr << "Failed to open corpus randomization file '"
			 << sRandomizationFileName << "' for writing." << endl;
		cerr << "Cannot continue. QUitting." << endl;
		return;
	}

	String sDate;
	getline (file, sDate);
	cout << "  - loading corpus randomization from date " << sDate << endl;

	String sLine;
	getline (file, sLine);

	Unigram_list_t lstIndices;
	sLine.Split (lstIndices);

	ITERATE_COLLECTION (Unigram_list_t, lstIndices, ite)
	{
		int iIndex = *ite;
		_dqIndices.push_back (iIndex);
	}

	int iIndices = _dqIndices.size ();
	if (iSubtreeCount != iIndices)
	{
		cerr << "ERROR : Number of subtrees in corpus (" << iSubtreeCount
			 << ") does not match number of randomized indices (" << iIndices 
			 << ") !  Quitting." << endl;
		return;
	}
}


//--------------------------------------------------------------------------------
void GetTrainingSections (BookSection* _pRoot, BookSection_list_t& _lstSections)
{
	deque <int> dqIndices;
	GetRandomizedIndices (_pRoot, dqIndices);
	int iIndices = dqIndices.size ();

	BookSection_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);

	double dStartRatio = Config::Get ("TOC_TRAIN_START_RATIO");
	double dEndRatio = Config::Get ("TOC_TRAIN_END_RATIO");
	int iStartSection = (int) (dStartRatio * dqIndices.size ());
	int iEndSection = (int) (dEndRatio * dqIndices.size ());

	cout << "  - Training sections : ";
	for (int i = 0; i < iIndices; ++ i)
	{
		if (i >= iEndSection)
			break;
		if (i < iStartSection)
			continue;

		cout << dqIndices [i] << ", ";
		_lstSections.push_back (dqAllSubtrees [dqIndices [i]]);
	}
	cout << endl;
}



//--------------------------------------------------------------------------------
void GetDecodingSections (BookSection* _pRoot, BookSection_list_t& _lstSections)
{
	deque <int> dqIndices;
	GetRandomizedIndices (_pRoot, dqIndices);
	int iIndices = dqIndices.size ();

	BookSection_dq_t	dqAllSubtrees;
	GetAllSubtrees (_pRoot, dqAllSubtrees);

	double dStartRatio = Config::Get ("TOC_DECODE_START_RATIO");
	double dEndRatio = Config::Get ("TOC_DECODE_END_RATIO");
	int iStartSection = (int) (dStartRatio * dqIndices.size ());
	int iEndSection = (int) (dEndRatio * dqIndices.size ());

	cout << "  - Decoding sections : ";
	for (int i = 0; i < iIndices; ++ i)
	{
		if (i >= iEndSection)
			break;
		if (i < iStartSection)
			continue;

		cout << dqIndices [i] << ", ";
		_lstSections.push_back (dqAllSubtrees [dqIndices [i]]);
	}
	cout << endl;
}



