#include "Config.h"
#include "Section.h"
#include "Tagger.h"
#include "Reader.h"
#include "Decoder.h"
#include "TreeDecoder.h"
#include "LanguageModel.h"
#include "Utils.h"
#include "FeatureEnumerator.h"
#include "nlp_string.h"
#include <sys/time.h>
#include <time.h>


void GetTrainingSections (Section* _pRoot, Section_list_t& _lstSections);
void GetDecodingSections (Section* _pRoot, Section_list_t& _lstSections);
void ComputeStatsForSections (Section_list_t& _lstSections, int _iStartDepth);


//--------------------------------------------------------------------------------
void ComputeStats (Section* _pRootSection, int _iStartDepth) 
{
	cout << "---------------------------------------------"  << endl;
	cout << " Statistics for all sections" << endl;
	cout << "---------------------------------------------"  << endl;
	Section_dq_t dqSections;
	Utils::SectionsAtDepth (dqSections, _pRootSection, _iStartDepth);
	
	Section_list_t	lstSections;
	ITERATE_COLLECTION (Section_dq_t, dqSections, ite)
		lstSections.push_back (*ite);
	ComputeStatsForSections (lstSections, _iStartDepth);

	cout << "---------------------------------------------"  << endl;
	cout << " Statistics for training sections" << endl;
	cout << "---------------------------------------------"  << endl;
	lstSections.clear ();
	GetTrainingSections (_pRootSection, lstSections);
	ComputeStatsForSections (lstSections, _iStartDepth);

	cout << "---------------------------------------------"  << endl;
	cout << " Statistics for testing sections" << endl;
	cout << "---------------------------------------------"  << endl;
	lstSections.clear ();
	GetDecodingSections (_pRootSection, lstSections);
	ComputeStatsForSections (lstSections, _iStartDepth);
}


//--------------------------------------------------------------------------------
void ComputeStatsForSections (Section_list_t& _lstSections, int _iStartDepth)
{
    int iTotalTrees =  _lstSections.size ();
    int iTotalTitles = 0;
    int iTotalTitleLength = 0;
    double dTotalAvgBranchingFactor = 0;
    int iMaxDepth = -1;
    int iTotalWords = 0;

    ITERATE_COLLECTION (Section_list_t, _lstSections, iteSubroot) 
	{
		Section* pSubroot = *iteSubroot;
		
		cout << "== SUB TREE =================================================" << endl;

		Section_list_t lstSections;
		Utils::GetInOrderSectionList (lstSections, pSubroot);
		
		int iBranches = 0;
		int iNonLeafNodes = 0;

		ITERATE_COLLECTION (Section_list_t, lstSections, iteSection) 
		{
			Section* pSection = *iteSection;

			cout << "ACT : " << pSection->p_Title->s_Text << endl;
			cout << "FLT : " << pSection->p_Title->s_Text << endl;

			int iTitleLength = pSection->p_Title->lst_Words.size();
			if (iTitleLength > 0) 
			{
				++ iTotalTitles;
				iTotalTitleLength += iTitleLength;
			}

			int iChildren = pSection->lst_Children.size();
			if (iChildren > 0) 
			{
				++ iNonLeafNodes;
				iBranches += iChildren;
			}

			int iDepth = pSection->GetDepth();
			if (iDepth > iMaxDepth)
				iMaxDepth = iDepth;
			
			iTotalWords += pSection->GetTotalUnigramOccurrences();
		}

		dTotalAvgBranchingFactor += iBranches / (double) iNonLeafNodes;
	}

    cout << "Max Depth: " << iMaxDepth + 1 << endl;
    cout << "Depth from SubRoots: " << iMaxDepth - _iStartDepth + 1 << endl;
    cout << "Number of Trees: " << iTotalTrees << endl;    
    cout << "Number of Titles: " << iTotalTitles << endl;
    cout << "Avg. Num of Titles: " << iTotalTitles / (double) iTotalTrees << endl;
    cout << "Avg Title Length: " << iTotalTitleLength / (double) iTotalTitles << endl;
    cout << "Avg Branching Factor: " << dTotalAvgBranchingFactor / (double) iTotalTrees << endl;
    cout << "Num of Words: " << iTotalWords << endl;
}

