#include "Config.h"
#include "Section.h"
#include "Tagger.h"
#include "Reader.h"
#include "Decoder.h"
#include "TreeDecoder.h"
#include "LanguageModel.h"
#include "Utils.h"
#include "FeatureEnumerator.h"
#include "nlp_string.h"
#include <sys/time.h>
#include <time.h>


void PrintTableOfContents (Section_dq_t dqTrainingSections, 
							Section_to_Title_map_t mapSectionToTitle, 
							bool _bTOC);


//---------------------------------------------------------------------
void TrainTOC (Model& _rModel, Section* _pRoot, Section_list_t& _lstSections)
{
	// load perceptron weights if necessary							
	int iDepth = Config::Get ("TOC_SECTION_DEPTH");
	String sTocWeightsFileName (Config::Get ("TOC_WEIGHTS_FILE_PATH"));
	String sFlatWeightsFileName (Config::Get ("WEIGHTS_FILE_PATH"));

	cout << "* Loading flat perceptron weights" << endl;
	_rModel.o_TitlePerceptron.LoadReadOnly (sFlatWeightsFileName);
	if (1 != (int) Config::Get ("RESET_WEIGHTS"))
	{
		cout << "* Loading TOC perceptron weights" << endl;
		_rModel.o_TocPerceptron.b_Hierarchical = true;
		_rModel.o_TocPerceptron.Load (sTocWeightsFileName);
	}
	else
		cout << "* Resetting perceptron weights" << endl;



	// train...														
	cout << "  - LM order : " << _rModel.lm_Lexical.i_Order << endl;
	cout << "  - Tag LM order : " << _rModel.lm_Tag.i_Order << endl;
	assert (_rModel.lm_Lexical.i_Order > 0);
	assert (_rModel.lm_Tag.i_Order > 0);

	cout << "* Training, depth : " << iDepth << endl;
	TreeDecoder trainer (_rModel);

	timeval tm_Start;
	timeval tm_End;
	gettimeofday (&tm_Start, NULL);

	int iEndSection = _lstSections.size ();
	int iTotalIterations = Config::Get ("TOC_TRAIN_ITERATIONS");
	for (int iIteration = 0; iIteration < iTotalIterations; ++ iIteration) 
	{
		int iSection = 0;
		ITERATE_COLLECTION (Section_list_t, _lstSections, ite) 
		{
			Section* pSection = *ite;

			// log iteration time...			
			gettimeofday (&tm_End, NULL);
			cout << (tm_End.tv_sec + tm_End.tv_usec / (double)1000000) -
					(tm_Start.tv_sec + tm_Start.tv_usec / (double)1000000);
			tm_Start = tm_End;

			// log iteration number & section...
			cout << "   [" << (iIteration + 1) << '/' << iTotalIterations 
				 << "][" << iSection << '/' << iEndSection 
				 << "] Training : " << pSection->p_Title->s_Text 
				 << " >> " << endl;

			// beam search !					
			trainer.BeamSearch (pSection, true);

			cout << endl;
			++ iSection;
		}

		String sTag;
		sTag << '[' << (iIteration + 1) << '/' << iTotalIterations << ']';

		_rModel.o_TocPerceptron.b_Hierarchical = true;
		_rModel.o_TocPerceptron.Save (sTocWeightsFileName, sTag);
	}
	return;
}



//---------------------------------------------------------------------
void DecodeTOC (Model& _rModel, Section* _pRoot, Section_list_t& _lstSections)
{
	String sTocWeightsFileName (Config::Get ("TOC_WEIGHTS_FILE_PATH"));
	String sFlatWeightsFileName (Config::Get ("WEIGHTS_FILE_PATH"));

	cout << "* Loading flat perceptron weights" << endl;
	_rModel.o_TitlePerceptron.LoadReadOnly (sFlatWeightsFileName);
	cout << "* Loading TOC perceptron weights" << endl;
	_rModel.o_TocPerceptron.b_Hierarchical = true;
	_rModel.o_TocPerceptron.LoadReadOnly (sTocWeightsFileName);


	int iDepth = Config::Get ("TOC_SECTION_DEPTH");
	cout << "* Decoding, depth : " << iDepth << endl;
	TreeDecoder decoder (_rModel);


	Section_to_Title_map_t mapSectionToTitle;
	Section_list_t	lstDecodedSections;

	int iEndSection = _lstSections.size ();
	int iSection = 0;
	ITERATE_COLLECTION (Section_list_t, _lstSections, ite) 
	{
		Section* pSection = *ite;

		cout << "   [" << iSection << '/' << iEndSection 
			 << "] Decoding : " << pSection->p_Title->s_Text << " >> " << endl;

		Section_to_Title_map_t mapDecode = decoder.BeamSearch (pSection, false);
		mapSectionToTitle.insert (mapDecode.begin (), mapDecode.end ());
		Utils::GetInOrderSectionList (lstDecodedSections, pSection);

		cout << endl;
		++ iSection;
	}

	Section_dq_t dqDecodeSections;
	ITERATE_COLLECTION (Section_list_t, lstDecodedSections, iteSection)
		dqDecodeSections.push_back (*iteSection);

	PrintTableOfContents (dqDecodeSections, mapSectionToTitle, true);
	return;
}
