#include <iostream>
#include "Reader.h"
#include "BookSection.h"


XMLCh*	AlgorithmsReader::px_TitleLiteral = NULL;
XMLCh*	AlgorithmsReader::px_BodyLiteral = NULL;
XMLCh*	AlgorithmsReader::px_SectionLiteral = NULL;
XMLCh*	AlgorithmsReader::px_RootLiteral = NULL;

XMLCh*	XMLReader::px_TitleLiteral = NULL;
XMLCh*	XMLReader::px_BodyLiteral = NULL;
XMLCh*	XMLReader::px_SectionLiteral = NULL;
XMLCh*	XMLReader::px_RootLiteral = NULL;
XMLCh*	XMLReader::px_TextLiteral = NULL;
XMLCh*	XMLReader::px_WordLiteral = NULL;
XMLCh*	XMLReader::px_NPLiteral = NULL;
XMLCh*	XMLReader::px_TagLiteral = NULL;
XMLCh*	XMLReader::px_CountLiteral = NULL;
XMLCh*	XMLReader::px_FOBSLiteral = NULL;
XMLCh*	XMLReader::px_FOBWLiteral = NULL;
XMLCh*	XMLReader::px_WordCountLiteral = NULL;
XMLCh*	XMLReader::px_SentenceCountLiteral = NULL;

//=================================================================
Reader::Reader (void)
{
}

Reader::~Reader (void)
{
}


Reader* Reader::CreateReader (const char* _zCorpusName)
{
    if ((0 == strcasecmp (_zCorpusName, DATASET_ALGORITHMS)) ||
		(0 == strcasecmp (_zCorpusName, DATASET_PHYSICS)) ||
		(0 == strcasecmp (_zCorpusName, DATASET_USHISTORY)))
        return new AlgorithmsReader;

	cerr << "Unknown dataset name : '" << _zCorpusName << "'." << endl;
    return NULL;
}


//=================================================================
AlgorithmsReader::AlgorithmsReader (void)
{
}

AlgorithmsReader::~AlgorithmsReader (void)
{
}


//=================================================================
BookSection* AlgorithmsReader::ParseFile (const char* _zFileName)
{

	XMLPlatformUtils::Initialize ();
	px_TitleLiteral = XMLString::transcode ("title");
	px_BodyLiteral = XMLString::transcode ("body");
	px_SectionLiteral = XMLString::transcode ("section");
	px_RootLiteral = XMLString::transcode ("root");
	
	XercesDOMParser* pParser = new XercesDOMParser ();

	pParser->setValidationScheme (XercesDOMParser::Val_Auto);
	pParser->setDoNamespaces (false);
	pParser->setDoSchema (false);
	pParser->setCreateEntityReferenceNodes (false);

	try
	{
		XMLCh* pxFileName = XMLString::transcode (_zFileName);
		LocalFileInputSource input (pxFileName);
		pParser->parse (input);
		delete pxFileName;

		if (0 != pParser->getErrorCount ())
		{
			std::cerr << "Encountered errors parsing XML file '"
					 << _zFileName << "'. Error count "
					 << pParser->getErrorCount () << std::endl;
			return NULL;
		}
	}
	catch (const DOMException& e)
	{
		std::cerr << "Exception while parsing XML file '"
			 << _zFileName << "'. ";
		if (NULL != e.msg)
		{
			char* zError = XMLString::transcode (e.msg);
			std::cerr << zError << std::endl;
			XMLString::release (&zError);
		}
		else
			std::cerr << "Error code : " << e.code << std::endl;
	}
	catch (const XMLException& e)
	{
		std::cerr << "Exception while parsing XML file '"
			 << _zFileName << "'. Exception : "
			 << e.getMessage () << std::endl;
		return NULL;
	}

	// traverse the XML document & create text section structure.
	DOMDocument* pDoc = pParser->getDocument ();
	if (NULL == pDoc) 
		return NULL;

	BookSection::i_TotalSections = 0;
	DOMTreeWalker* pWalker = pDoc->createTreeWalker (pDoc, DOMNodeFilter::SHOW_ALL, NULL, true);
	pWalker->firstChild ();
	BookSection* pRootSection = AlgorithmsReader::Parse (pWalker);
	pRootSection->BuildRootStructures ();

	BookSection_list_t* pAllSections = pRootSection->GetAllSections ();
	BookSection* pPreviousSection = NULL;

	BookSection_list_t::iterator	ite;
	for (ite = pAllSections->begin ();
		ite != pAllSections->end ();
		++ ite)
	{
		BookSection* pSection = *ite;
		pSection->p_PreviousSection = pPreviousSection;
		pPreviousSection = pSection;
	}
	// deleting the parse is supposed to release all memory allocations associated 	
	// with it - including the DOMDocument & the DOMTreeWalker...					
	delete pParser;

	return pRootSection;
}


//=================================================================
BookSection* AlgorithmsReader::Parse (DOMTreeWalker* _pWalker)
{
	String sTitle;
	String sBody;

	++ BookSection::i_TotalSections;
	BookSection* pSection = new BookSection;

	DOMNode* pChildNode = _pWalker->firstChild ();
	while (NULL != pChildNode)
	{
		const XMLCh* pxNodeName = pChildNode->getNodeName ();
		if (true == XMLString::equals (pxNodeName, px_TitleLiteral))
		{
			const XMLCh* pxTitleText = pChildNode->getTextContent ();
			char* pTitleText = XMLString::transcode (pxTitleText);
			((string&) sTitle) = pTitleText;
		}
		if (true == XMLString::equals (pxNodeName, px_BodyLiteral))
		{
			const XMLCh* pxBodyText = pChildNode->getTextContent ();
			char* pBodyText = XMLString::transcode (pxBodyText);
			((string&) sBody) = pBodyText;
		}
		if ((true == XMLString::equals (pxNodeName, px_SectionLiteral)) ||
			(true == XMLString::equals (pxNodeName, px_RootLiteral)))
		{
			BookSection* pChildSection = AlgorithmsReader::Parse (_pWalker);
			pSection->AddChild (pChildSection);
		}

		pChildNode = _pWalker->nextSibling ();
	}
	_pWalker->parentNode ();

	pSection->SetText (sTitle, sBody);
	
	return pSection;
}
                                                                                                                                                                                              
//=================================================================
XMLReader::XMLReader (void)
{
}

XMLReader::~XMLReader (void)
{
}


//=================================================================
Section* XMLReader::ParseFile (const char* _zFileName)
{

	XMLPlatformUtils::Initialize ();
	px_TitleLiteral = XMLString::transcode ("title");
	px_BodyLiteral = XMLString::transcode ("body");
	px_SectionLiteral = XMLString::transcode ("section");
	px_RootLiteral = XMLString::transcode ("root");
	px_TextLiteral = XMLString::transcode ("text");
	px_WordLiteral = XMLString::transcode ("word");
	px_NPLiteral = XMLString::transcode ("np");

	px_TagLiteral = XMLString::transcode ("tag");
	px_CountLiteral = XMLString::transcode ("count");
	px_FOBSLiteral = XMLString::transcode ("fobs");
	px_FOBWLiteral = XMLString::transcode ("fobw");
	px_WordCountLiteral = XMLString::transcode ("words");
	px_SentenceCountLiteral = XMLString::transcode ("sentences");
	
	XercesDOMParser* pParser = new XercesDOMParser ();

	pParser->setValidationScheme (XercesDOMParser::Val_Auto);
	pParser->setDoNamespaces (false);
	pParser->setDoSchema (false);
	pParser->setCreateEntityReferenceNodes (false);

	try
	{
		XMLCh* pxFileName = XMLString::transcode (_zFileName);
		LocalFileInputSource input (pxFileName);
		pParser->parse (input);
		delete pxFileName;

		if (0 != pParser->getErrorCount ())
		{
			std::cerr << "Encountered errors parsing XML file '"
					 << _zFileName << "'. Error count "
					 << pParser->getErrorCount () << std::endl;
			return NULL;
		}
	}
	catch (const DOMException& e)
	{
		std::cerr << "Exception while parsing XML file '"
			 << _zFileName << "'. ";
		if (NULL != e.msg)
		{
			char* zError = XMLString::transcode (e.msg);
			std::cerr << zError << std::endl;
			XMLString::release (&zError);
		}
		else
			std::cerr << "Error code : " << e.code << std::endl;
	}
	catch (const XMLException& e)
	{
		std::cerr << "Exception while parsing XML file '"
			 << _zFileName << "'. Exception : "
			 << e.getMessage () << std::endl;
		return NULL;
	}

	// traverse the XML document & create text section structure.
	DOMDocument* pDoc = pParser->getDocument ();
	if (NULL == pDoc) 
		return NULL;

	Section::i_TotalSections = 0;
	DOMTreeWalker* pWalker = pDoc->createTreeWalker (pDoc, DOMNodeFilter::SHOW_ALL, NULL, true);
	pWalker->firstChild ();
	Section* pRootSection = XMLReader::Parse (pWalker);
	pRootSection->BuildRootStructures ();

	Section_list_t* pAllSections = pRootSection->GetAllSections ();
	Section* pPreviousSection = NULL;

	Section_list_t::iterator	ite;
	for (ite = pAllSections->begin ();
		ite != pAllSections->end ();
		++ ite)
	{
		Section* pSection = *ite;
		pSection->p_PreviousSection = pPreviousSection;
		pPreviousSection = pSection;
	}
	// deleting the parse is supposed to release all memory allocations associated 	
	// with it - including the DOMDocument & the DOMTreeWalker...					
	delete pParser;

	return pRootSection;
}


//=================================================================
Section* XMLReader::Parse (DOMTreeWalker* _pWalker)
{
	String sTitle;
	String sBody;

	++ Section::i_TotalSections;
	Section* pSection = new Section;

	DOMNode* pChildNode = _pWalker->firstChild ();
	while (NULL != pChildNode)
	{
		const XMLCh* pxNodeName = pChildNode->getNodeName ();
		if (true == XMLString::equals (pxNodeName, px_TitleLiteral))
			ParseTitleNode (pSection, _pWalker);

		if (true == XMLString::equals (pxNodeName, px_BodyLiteral))
			ParseBodyNode (pSection, _pWalker);

		if (true == XMLString::equals (pxNodeName, px_WordCountLiteral))
		{
			const XMLCh* pxCount = pChildNode->getTextContent ();
			String sCount ((char*) XMLString::transcode (pxCount));
			pSection->i_TotalUnigrams = sCount;
		}

		if (true == XMLString::equals (pxNodeName, px_SentenceCountLiteral))
		{
			const XMLCh* pxCount = pChildNode->getTextContent ();
			String sCount ((char*) XMLString::transcode (pxCount));
			pSection->l_TotalSentences = sCount;
		}

		if (true == XMLString::equals (pxNodeName, px_NPLiteral))
		{
			DOMNamedNodeMap* pMap = pChildNode->getAttributes ();
			assert (NULL != pMap);

			const XMLCh* pxValue;

			DOMNode* pText = pMap->getNamedItem (px_TextLiteral);
			assert (NULL != pText);
			pxValue = pText->getNodeValue ();
			String sNP ((char*) XMLString::transcode (pxValue));

			DOMNode* pCount = pMap->getNamedItem (px_CountLiteral);
			assert (NULL != pCount);
			pxValue = pCount->getNodeValue ();
			String sCount ((char*) XMLString::transcode (pxValue));

			pSection->map_NP_to_Corpus_Count.insert (make_pair (sNP, (long) sCount));
		}
		if ((true == XMLString::equals (pxNodeName, px_SectionLiteral)) ||
			(true == XMLString::equals (pxNodeName, px_RootLiteral)))
		{
			Section* pChildSection = XMLReader::Parse (_pWalker);
			pSection->AddChild (pChildSection);
		}

		pChildNode = _pWalker->nextSibling ();
	}
	_pWalker->parentNode ();

	return pSection;
}


//=================================================================
void XMLReader::ParseTitleNode (Section* _pSection, DOMTreeWalker* _pWalker)
{
	DOMNode* pChildNode = _pWalker->firstChild ();
	while (NULL != pChildNode)
	{
		const XMLCh* pxNodeName = pChildNode->getNodeName ();
		if (true == XMLString::equals (pxNodeName, px_TextLiteral))
		{
			const XMLCh* pxTitleText = pChildNode->getTextContent ();
			char* pTitleText = XMLString::transcode (pxTitleText);
			_pSection->SetTitleText (pTitleText);
		}
		if (true == XMLString::equals (pxNodeName, px_WordLiteral))
		{
			DOMNamedNodeMap* pMap = pChildNode->getAttributes ();
			assert (NULL != pMap);

			const XMLCh* pxValue;

			DOMNode* pText = pMap->getNamedItem (px_TextLiteral);
			assert (NULL != pText);
			pxValue = pText->getNodeValue ();
			String sWord ((char*) XMLString::transcode (pxValue));

			DOMNode* pTag = pMap->getNamedItem (px_TagLiteral);
			assert (NULL != pTag);
			pxValue = pTag->getNodeValue ();
			String sTag ((char*) XMLString::transcode (pxValue));

			Word* pWord = new Word (sWord);
			pWord->s_POSTag = sTag;

			_pSection->AddTitleWord (pWord);
		}

		pChildNode = _pWalker->nextSibling ();
	}
	_pWalker->parentNode ();
}


//=================================================================
void XMLReader::ParseBodyNode (Section* _pSection, DOMTreeWalker* _pWalker)
{
	DOMNode* pChildNode = _pWalker->firstChild ();
	while (NULL != pChildNode)
	{
		const XMLCh* pxNodeName = pChildNode->getNodeName ();
		if (true == XMLString::equals (pxNodeName, px_WordLiteral))
			ParseWordNode (_pSection, pChildNode);
		if (true == XMLString::equals (pxNodeName, px_NPLiteral))
		{
			DOMNamedNodeMap* pMap = pChildNode->getAttributes ();
			assert (NULL != pMap);

			const XMLCh* pxValue;

			DOMNode* pText = pMap->getNamedItem (px_TextLiteral);
			assert (NULL != pText);
			pxValue = pText->getNodeValue ();
			String sNP ((char*) XMLString::transcode (pxValue));

			DOMNode* pCount = pMap->getNamedItem (px_CountLiteral);
			assert (NULL != pCount);
			pxValue = pCount->getNodeValue ();
			String sCount ((char*) XMLString::transcode (pxValue));

			_pSection->map_NP_to_Count.insert (make_pair (sNP, (long) sCount));
		}

		pChildNode = _pWalker->nextSibling ();
	}
	_pWalker->parentNode ();
}



//=================================================================
void XMLReader::ParseWordNode (Section* _pSection, DOMNode* _pNode)
{
	DOMNamedNodeMap* pMap = _pNode->getAttributes ();
	assert (NULL != pMap);

	const XMLCh* pxValue;

	DOMNode* pText = pMap->getNamedItem (px_TextLiteral);
	assert (NULL != pText);
	pxValue = pText->getNodeValue ();
	String sWord ((char*) XMLString::transcode (pxValue));

	DOMNode* pTag = pMap->getNamedItem (px_TagLiteral);
	assert (NULL != pTag);
	pxValue = pTag->getNodeValue ();
	String sTag ((char*) XMLString::transcode (pxValue));

	int lCount = 0;
	DOMNode* pCount = pMap->getNamedItem (px_CountLiteral);
	if (NULL != pCount)
	{
		pxValue = pCount->getNodeValue ();
		String sCount ((char*) XMLString::transcode (pxValue));
		lCount = sCount;	
	}
	int lFOBS = -1;
	DOMNode* pFOBS = pMap->getNamedItem (px_FOBSLiteral);
	if (NULL != pFOBS)
	{
		pxValue = pFOBS->getNodeValue ();
		String sFOBS ((char*) XMLString::transcode (pxValue));
		lFOBS = sFOBS;
	}
	int lFOBW = -1;
	DOMNode* pFOBW = pMap->getNamedItem (px_FOBWLiteral);
	if (NULL != pFOBW)
	{
		pxValue = pFOBW->getNodeValue ();
		String sFOBW ((char*) XMLString::transcode (pxValue));
		lFOBW = sFOBW;
	}


	Word* pWord = new Word (sWord);
	pWord->s_POSTag = sTag;

	_pSection->AddBodyWord (pWord, lCount, lFOBW, lFOBS);
}
