#ifndef __TITLE_GEN_READER__
#define __TITLE_GEN_READER__

#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include "nlp_string.h"
#include "Section.h"
#include "BookSection.h"

#ifdef XERCES_CPP_NAMESPACE_USE
XERCES_CPP_NAMESPACE_USE
#endif

#define DATASET_ALGORITHMS "algorithms"
#define DATASET_PHYSICS "physics"
#define DATASET_USHISTORY "us_history"



//=================================================================
class Reader
{
	private:
		virtual BookSection* Parse (DOMTreeWalker* _pWalker) = 0;	
		
	public:
		Reader (void);
		virtual ~Reader (void);

        static Reader* CreateReader (const char* _zCorpusName);
		virtual BookSection* ParseFile (const char* _zFileName) = 0;
};
                                                                                               


//=================================================================
class AlgorithmsReader : public Reader
{
	private:
		static XMLCh* px_TitleLiteral;
		static XMLCh* px_BodyLiteral;
		static XMLCh* px_SectionLiteral;
		static XMLCh* px_RootLiteral;
		
		virtual BookSection* Parse (DOMTreeWalker* _pWalker);

	public:
		AlgorithmsReader (void);
		virtual ~AlgorithmsReader (void);
	
		virtual BookSection* ParseFile (const char* _zFileName);
};



//=================================================================
class XMLReader
{
	private:
		static XMLCh* px_TitleLiteral;
		static XMLCh* px_BodyLiteral;
		static XMLCh* px_SectionLiteral;
		static XMLCh* px_RootLiteral;
		static XMLCh* px_TextLiteral;
		static XMLCh* px_WordLiteral;
		static XMLCh* px_NPLiteral;
		static XMLCh* px_TagLiteral;
		static XMLCh* px_CountLiteral;
		static XMLCh* px_FOBSLiteral;
		static XMLCh* px_FOBWLiteral;
		static XMLCh* px_WordCountLiteral;
		static XMLCh* px_SentenceCountLiteral;

		
		virtual Section* Parse (DOMTreeWalker* _pWalker);
		void ParseTitleNode (Section* _pSection, DOMTreeWalker* _pWalker);
		void ParseBodyNode (Section* _pSection, DOMTreeWalker* _pWalker);
		void ParseWordNode (Section* _pSection, DOMNode* _pNode);

	public:
		XMLReader (void);
		virtual ~XMLReader (void);
	
		virtual Section* ParseFile (const char* _zFileName);
};

#endif
