#include "Tagger.h"
#include "Utils.h"
#include <iostream>
#include <fstream>
#include "Config.h"
#include "Utils.h"
using namespace std;


Tagger::Tagger (void)
{
}

Tagger::~Tagger (void)
{
}



//--------------------------------------------------------------------------------
void Tagger::WriteTagInputBySection (BookSection* _pSection, ofstream& _rFile)
{
    _rFile << Config::Get ("TAGGER_DOCUMENT_START_TAG") << endl;

    _rFile << _pSection->p_Title->s_Text << endl;

    Sentence_list_t::iterator iteSentence ;
    for (iteSentence = _pSection->lst_Sentences.begin (); 
        iteSentence  != _pSection->lst_Sentences.end (); 
        ++ iteSentence)
	{
	    Sentence* pSentence = *iteSentence;
	    _rFile << pSentence->s_Text << endl;
	}

    _rFile << Config::Get ("TAGGER_DOCUMENT_END_TAG") << endl;
}


//--------------------------------------------------------------------------------
void Tagger::ParseTaggedOutput (const char* _zFileName, BookSection_list_t& _rlstSections)
{
    ifstream file (_zFileName);
	if (false == file.is_open ())
	{
		cerr << "Failed to open file '" << _zFileName << "' to read tagged text." << endl;
		return;
	}
	
    String sDocumentStartTag = Config::Get ("TAGGER_DOCUMENT_START_TAG");

    BookSection_list_t::iterator iteSection = _rlstSections.begin ();
    while ((false == file.eof ()) && 
           (iteSection != _rlstSections.end ()))
    {
        String sLine;
        getline (file, sLine);

        if (sLine.find (sDocumentStartTag) != string::npos)
        {
            ParseTaggedOutputBySection (file, *iteSection);
			CheckForUntaggedWords (*iteSection);
            ++iteSection;
        }
    }   
}



//--------------------------------------------------------------------------------
void Tagger::CheckForUntaggedWords (BookSection* _pSection)
{
	ITERATE_COLLECTION (Word_dq_t, _pSection->dq_UniqueWords, ite)
	{
		Word* pWord = *ite;
		if (pWord->s_POSTag == "")
			cerr << "[WARNING] Tag not present for word '" << pWord->s_Word << "'." << endl;
	}
}


//--------------------------------------------------------------------------------
void Tagger::ParseTaggedOutputBySection (ifstream& _rFile, BookSection* _pSection) 
{
	// Tag the title
	String sLine;
	getline (_rFile, sLine);
	SetWordTags (sLine, _pSection->p_Title);

    String sDocumentEndTag = Config::Get ("TAGGER_DOCUMENT_END_TAG");

    // Tag the body
    Sentence_list_t::iterator iteSentence = _pSection->lst_Sentences.begin (); 
    while ((false == _rFile.eof ()) && 
           (iteSentence != _pSection->lst_Sentences.end ()))
    {
        String sLine;
        getline (_rFile, sLine);

		if (sLine.find (sDocumentEndTag) != string::npos)
            break;

        SetWordTags (sLine, *iteSentence);
        ++iteSentence;
    }

    // Create unique word list
    _pSection->CreateUniqueWordList ();
}



//--------------------------------------------------------------------------------
void Tagger::SetWordTags (String& _sParsedSentence, Sentence* _pSentence)
{
    if (0 == _pSentence->lst_Words.size ())
		return;

    String sCorpus = Config::Get ("DATASET");

    list<String> lst_WordTags;
    _sParsedSentence.Split (lst_WordTags);

    Word_list_t::iterator iteSentenceWord = _pSentence->lst_Words.begin ();
    Word* pSentenceWord = *iteSentenceWord;

    ITERATE_COLLECTION (Unigram_list_t, lst_WordTags, iteWordTags) 
    {
        list <String> lst_WordTagPair;
        iteWordTags->Split (lst_WordTagPair, '_', 1);

        if (lst_WordTagPair.size () != 2)
		{
			cerr << "[WARNING] Tag not found for word '" << lst_WordTagPair.front () 
				 << "'. Text : '" << *iteWordTags << endl;
			cout << lst_WordTags << endl;
			cout << _pSentence->lst_Words << endl;
            continue;
		}

        list <String>::iterator iteWordTagPair = lst_WordTagPair.begin ();
        String sWord = *iteWordTagPair; 
        ++iteWordTagPair;
        String sTag = *iteWordTagPair;
        sWord.LowerCase ();

        if (sWord == pSentenceWord->s_Word)
        {
            if (sCorpus == "algorithms")
            {
                if ((sTag == "NNS") &&
                    ((sWord == "shows") || (sWord == "computes") ||
                     (sWord == "determines") || (sWord == "lies") ||
                     (sWord == "straddles") || (sWord == "varies") ||
                     (sWord == "approaches") || (sWord == "oscillates") ||
                     (sWord == "investigates")))
                    sTag = "VBZ";
                else if ((sWord == "show") && ((sTag == "NN") || (sTag == "NNP")))
                    sTag = "VB";
                else if ((sWord == "in") && (sTag == "NNP"))
                    sTag = "IN";
                else if ((sWord == "that") && (sTag == "IN"))
                    sTag = "WDT";
                else if (sWord == "9999")
                    sTag = "CD";
            }

			pSentenceWord->s_POSTag = sTag;
			++ iteSentenceWord;

			if (iteSentenceWord == _pSentence->lst_Words.end ())
				return;

			pSentenceWord = *iteSentenceWord;
        }
		else
		{
			cout << "[WARNING] mismatching word pair : (sentence word) '" << pSentenceWord->s_Word
				 << "'  (tagged word) '" << sWord << "'." << endl;
			cout << lst_WordTags << endl;
			cout << _pSentence->lst_Words << endl;
		}
    }
}



//--------------------------------------------------------------------------------
void Tagger::TagCorpus (BookSection* _pRootSection) 
{
    String sTaggerInputPath = Config::GetTempFileName ("TAGGER_INPUT_PATH");
    String sTaggerOutputPath = Config::Get("TAGGER_OUTPUT_PATH");

    BookSection_list_t* plstAllSections = _pRootSection->GetAllSections ();

    // Create Input File
    {
		ofstream fileout (sTaggerInputPath);
	
        BookSection_list_t::iterator ite;
        for (ite = plstAllSections->begin (); 
            ite != plstAllSections->end (); 
            ++ ite)
	    {
			//cout << "Writing tag input for section " << (*ite)->p_Title->s_Text << endl;
			WriteTagInputBySection (*ite, fileout);
	    }
	
        fileout.flush ();
        fileout.close ();
    }

    // Run tagger
    {
        String sTaggerCommandLine;
        sTaggerCommandLine << Config::Get ("TAGGER_EXECUTABLE") 
                           << " < " << sTaggerInputPath 
                           << " > " << sTaggerOutputPath;
		cout << sTaggerCommandLine << endl;
        system (sTaggerCommandLine);
    }

    // Parse output
    ParseTaggedOutput (sTaggerOutputPath, *plstAllSections);
}



//--------------------------------------------------------------------------------
void Tagger::TagFromFile (BookSection* _pRootSection)
{
    String sTaggerOutputPath = Config::Get("TAGGER_OUTPUT_PATH");
    BookSection_list_t* plstAllSections = _pRootSection->GetAllSections ();
    ParseTaggedOutput (sTaggerOutputPath, *plstAllSections);
}



//--------------------------------------------------------------------------------
#define map_has_key(_map,_key)  (_map.find (_key) != _map.end ())  

void Tagger::TagTitlesFromText (BookSection* _pRootSection)
{
    Word_to_TagCount_map_t mapWordToTagCount;
    BookSection_list_t* plstAllSections = _pRootSection->GetAllSections ();

    // Build word to tag counts from body text
    for (BookSection_list_t::iterator iteSection = plstAllSections->begin (); 
        iteSection  != plstAllSections->end (); 
        ++ iteSection)
    {
	    BookSection* pSection = *iteSection;

		Sentence_list_t::iterator	iteSentence;
		for (iteSentence = pSection->lst_Sentences.begin ();
			iteSentence != pSection->lst_Sentences.end ();
			++ iteSentence)
		{
			Sentence* pSentence = *iteSentence;
			Word_list_t::iterator iteWord;
			for (iteWord = pSentence->lst_Words.begin (); 
				iteWord != pSentence->lst_Words.end (); 
				++ iteWord)
			{
				Word* pWord = *iteWord;

				Tag_to_Count_map_t* pMap = NULL;
				Word_to_TagCount_map_t::iterator	iteMap;
				iteMap = mapWordToTagCount.find (pWord->s_Word);
				if (iteMap == mapWordToTagCount.end ())
				{
					pair <Word_to_TagCount_map_t::iterator, bool> pair;
					pair = mapWordToTagCount.insert (make_pair (pWord->s_Word, Tag_to_Count_map_t ()));
					pMap = &pair.first->second;
				}
				else
					pMap = &iteMap->second;

				Tag_to_Count_map_t::iterator	iteTag;
				iteTag = pMap->find (pWord->s_POSTag);
				if (iteTag == pMap->end ())
					pMap->insert (make_pair (pWord->s_POSTag, 1));
				else
					++ iteTag->second;
			}
		}
	}    

    
    // Get majority POS tag for each word
    Word_to_Tag_map_t mapWordToMaxTag;
    ITERATE_COLLECTION( Word_to_TagCount_map_t, mapWordToTagCount, iteWordToTagMap )
    {
        String sWord = iteWordToTagMap->first;
        Tag_to_Count_map_t map_TagToCount = iteWordToTagMap->second;

        String sMaxTag;
        int iMaxTagCount = -1;

		ITERATE_COLLECTION( Tag_to_Count_map_t, map_TagToCount,  iteTagToCountMap )
        {
            String sTag = iteTagToCountMap->first;
            int iCount = iteTagToCountMap->second;

            if (iMaxTagCount <= iCount) 
            {
                iMaxTagCount = iCount;
                sMaxTag = sTag;
            }
        }
        mapWordToMaxTag [sWord] = sMaxTag;
    }


    // Tag titles
    for (BookSection_list_t::iterator iteSection = plstAllSections->begin (); 
        iteSection  != plstAllSections->end (); 
        ++ iteSection)
	{
        BookSection* pSection = *iteSection;

        Word_list_t& rlstWords = pSection->p_Title->lst_Words;
        for (Word_list_t::iterator iteTitleWords = rlstWords.begin ();
            iteTitleWords != rlstWords.end ();
            ++iteTitleWords) 
        {
            Word* pTitleWord = *iteTitleWords;
			Word_to_Tag_map_t::iterator	iteTag;
			iteTag = mapWordToMaxTag.find (pTitleWord->s_Word);
			if (iteTag != mapWordToMaxTag.end ())
				pTitleWord->s_POSTag = iteTag->second;
        }
    }
}
