#include "TreeDecoder.h"
#include "Decoder.h"
#include "Config.h"
#include "Utils.h"
#include <limits>

int SectionTitle::i_TagNGramSize = 0;
int TreePath::i_InitialStartWords = 0;
Word* TreePath::p_StartWord = NULL;

Word* TreeDecoder::p_EndWord = NULL;
int TreeDecoder::i_Order = 0;
bool TreeDecoder::b_HierarchicalDecoding = true;
bool TreeDecoder::b_LastWordEvaluation = false;
bool TreeDecoder::b_Complete = false;



//-----------------------------------------------------------
SectionTitle::SectionTitle (SectionTitle& _rOld) 
{
	dq_Words = _rOld.dq_Words;
	s_Title = _rOld.s_Title;
	s_TitleStems = _rOld.s_TitleStems;
	s_ContentWords = _rOld.s_ContentWords;
	set_TitleStems = _rOld.set_TitleStems;
	map_StemCounts = _rOld.map_StemCounts;
	map_TagNgramCounts = _rOld.map_TagNgramCounts;
	p_Section = _rOld.p_Section;
}


//-----------------------------------------------------------
SectionTitle::SectionTitle (Section* _pSection, Path* _pSectionPath) 
{
	p_Section = _pSection;
	dq_Words = _pSectionPath->dq_Words;

	int iNgramSize = i_TagNGramSize - 1;
	int iTitleLength = dq_Words.size ();
	for (int i = 0; i < iTitleLength; ++ i)
	{
		Word* pWord = dq_Words [i];

		if ((true == pWord->b_StartWord) || (true == pWord->b_EndWord))
			continue;

		s_Title << pWord->s_Word << ' ';

		// Add to list of title stems	
		if (true == pWord->b_IsContentWord)
		{
			set_TitleStems.insert (pWord->s_Stem);
			s_ContentWords << pWord->s_Word << '|';
		}

		// Add to stem counts			
		Unigram_to_Count_map_t::iterator iteStemCount;
		iteStemCount = map_StemCounts.find (pWord->s_Stem);
		if (iteStemCount == map_StemCounts.end ())
			map_StemCounts.insert (make_pair (pWord->s_Stem, 1));
		else
			++iteStemCount;


		// Add tag trigram	iNgramSize is 1 less than actual...
		if (i >= iNgramSize + 1) 
		{
			String sTagNgram;
			sTagNgram	<< dq_Words [i - iNgramSize]->s_POSTag << ' ' 
						<< dq_Words [i - iNgramSize + 1]->s_POSTag << ' ' 
						<< dq_Words [i - iNgramSize + 2]->s_POSTag;
			Unigram_to_Count_map_t::iterator iteNgramCount;
			iteNgramCount = map_TagNgramCounts.find (sTagNgram);
			if (iteNgramCount == map_TagNgramCounts.end ())
				map_TagNgramCounts.insert (make_pair (sTagNgram, 1));
			else
				++iteNgramCount;

			if (i == iTitleLength - 1)
			{
				String sTagNgram;
				sTagNgram	<< dq_Words [i - iNgramSize + 1]->s_POSTag << ' ' 
							<< dq_Words [i - iNgramSize + 2]->s_POSTag << ' ' 
							<< Word::p_EndWord->s_POSTag;
				Unigram_to_Count_map_t::iterator iteNgramCount;
				iteNgramCount = map_TagNgramCounts.find (sTagNgram);
				if (iteNgramCount == map_TagNgramCounts.end ())
					map_TagNgramCounts.insert (make_pair (sTagNgram, 1));
				else
					++iteNgramCount;
			}
		}
	}

	ITERATE_COLLECTION (Unigram_set_t, set_TitleStems, ite)
		s_TitleStems << *ite << '|';
}


//-----------------------------------------------------------
TreePath::TreePath (void)
{
	d_Score = -1000000000;
	d_SectionPathScoreTotal = 0;
	p_FeatureVector = NULL;
	b_YGoodMatch = true;
	p_CurrentSection = NULL;
	p_LastSectionTitle = NULL;
	b_HasDuplicateTitle = false;

	memset (arr_PathFeatureTotals, 0, FEATURE_COUNT * sizeof (double));
	memset (arr_PathFeatureCounts, 0, FEATURE_COUNT * sizeof (long));
}


//-----------------------------------------------------------
TreePath::TreePath (TreePath& _rOld)
{
	d_Score = _rOld.d_Score;
	d_SectionPathScoreTotal = _rOld.d_SectionPathScoreTotal;
	b_YGoodMatch = _rOld.b_YGoodMatch;
	p_CurrentSection = _rOld.p_CurrentSection;
	b_LastWordIsNotInSection = _rOld.b_LastWordIsNotInSection;
	b_HasDuplicateTitle = _rOld.b_HasDuplicateTitle;

	if (NULL != _rOld.p_FeatureVector)
		p_FeatureVector = new FeatureVector (*_rOld.p_FeatureVector);
	else
		p_FeatureVector = NULL;


	ITERATE_COLLECTION (SectionTitle_dq_t, _rOld.dq_SectionTitles, iteStore) 
	{
		SectionTitle* pNewTitle = new SectionTitle (**iteStore);
		dq_SectionTitles.push_back (pNewTitle);
		hmp_SectionToTitle.insert (make_pair ((long) pNewTitle->p_Section, 
													pNewTitle));
	}

	ITERATE_COLLECTION (SectionToTitle_hmp_t, hmp_SectionToTitle, iteSibling)
	{
		Section* pSection = (Section*) iteSibling->first;
		SectionToTitleDQ_hmp_t::iterator	iteParent;
		iteParent = hmp_SectionToTitleStoreDQ.find ((long) pSection->p_Parent);
		if (iteParent == hmp_SectionToTitleStoreDQ.end ())
		{
			pair <SectionToTitleDQ_hmp_t::iterator, bool>	pairInsert;
			pairInsert = hmp_SectionToTitleStoreDQ.insert (
										make_pair ((long) pSection->p_Parent, 
													SectionTitle_dq_t ())); 
			iteParent = pairInsert.first;
		}
		iteParent->second.push_back (iteSibling->second);
	}
	
	hmp_TitleStems = _rOld.hmp_TitleStems;
	hmp_ContentWords = _rOld.hmp_ContentWords;


	for (int i = 0; i < FEATURE_COUNT; ++ i)
	{
		arr_PathFeatureTotals [i] = _rOld.arr_PathFeatureTotals [i];
		arr_PathFeatureCounts [i] = _rOld.arr_PathFeatureCounts [i];
	}
}


//-----------------------------------------------------------
void TreePath::StartNewSection (Section* _pSection)
{
	p_CurrentSection = _pSection;
}


//-----------------------------------------------------------
SectionTitle* TreePath::GetSectionTitle (Section* _pSection) 	// was GetTitleStore
{
	SectionToTitle_hmp_t::iterator ite;
	ite = hmp_SectionToTitle.find ((long) _pSection);
	return (ite == hmp_SectionToTitle.end ()) ? NULL : ite->second;
}


//-----------------------------------------------------------
SectionTitle_dq_t* TreePath::GetPreviousSiblings (Section* _pSection)
{
	SectionToTitleDQ_hmp_t::iterator ite;
	ite = hmp_SectionToTitleStoreDQ.find ((long) _pSection);
	if (ite == hmp_SectionToTitleStoreDQ.end ())
		return NULL;

	return &(ite->second);
}


//-----------------------------------------------------------
TreePath::~TreePath (void)
{
	if (NULL != p_FeatureVector) 
		delete p_FeatureVector;

	ITERATE_COLLECTION (SectionTitle_dq_t, dq_SectionTitles, ite)
		delete *ite;
	dq_SectionTitles.clear ();

	hmp_SectionToTitle.clear ();
}


//-----------------------------------------------------------
void TreePath::Grow (Path* _pSectionPath, 
					double _dScore, 
					FeatureVector* _pFeatureVector)
{
	d_Score = _dScore;
	d_SectionPathScoreTotal += _pSectionPath->d_Score;

	b_YGoodMatch &= _pSectionPath->b_YGoodMatch;

	if (p_FeatureVector != _pFeatureVector)
	{
		if (NULL != p_FeatureVector)
			delete p_FeatureVector;
		p_FeatureVector = _pFeatureVector;
	}


	// create a new SectionTitle object from the SectionPath & add to the		
	// relevant local structures...												
	p_LastSectionTitle = new SectionTitle (p_CurrentSection, _pSectionPath);
	dq_SectionTitles.push_back (p_LastSectionTitle);
	hmp_SectionToTitle.insert (make_pair ((long) p_CurrentSection, p_LastSectionTitle));

	
	// add the new SectionTitle to it's parent section's dq of child titles...	
	if (NULL != p_CurrentSection->p_Parent)
	{
		SectionToTitleDQ_hmp_t::iterator	iteParent;
		iteParent = hmp_SectionToTitleStoreDQ.find ((long) p_CurrentSection->p_Parent);
		if (iteParent == hmp_SectionToTitleStoreDQ.end ())
		{
			pair <SectionToTitleDQ_hmp_t::iterator, bool>	pairInsert;
			pairInsert = hmp_SectionToTitleStoreDQ.insert (
										make_pair ((long) p_CurrentSection->p_Parent, 
													SectionTitle_dq_t ())); 
			iteParent = pairInsert.first;
		}
		iteParent->second.push_back (p_LastSectionTitle);
	}

	// add the new section title's concaternated word stems to the path title 	
	// stem map. This is used to check if two titles are identical by their		
	// content word stems...													
	hmp_TitleStems.insert (make_pair (p_LastSectionTitle->s_TitleStems, true));

	// the duplicate check below has to be done before inserting the content words to the map !
	b_HasDuplicateTitle |= (hmp_ContentWords.find (p_LastSectionTitle->s_ContentWords) != hmp_ContentWords.end ());
	hmp_ContentWords.insert (make_pair (p_LastSectionTitle->s_ContentWords, true));
}
 

//-----------------------------------------------------------
bool TreePath::operator< (TreePath& _rRight)
{
	return (d_Score < _rRight.d_Score);
}


//-----------------------------------------------------------
bool TreePath::IsYGood (void)
{
	return b_YGoodMatch;
}


//-----------------------------------------------------------
void TreePath::AccumulatePathFeatures (void)
{
	if (NULL == p_FeatureVector)
		return;

	for (int i = 0; i < BASE_FEATURE_COUNT; ++ i) 
	{
		arr_PathFeatureTotals [AVERAGE_FEATURE_OFFSET + i] 
							+= p_FeatureVector->arr_Features [i];
		++ arr_PathFeatureCounts [AVERAGE_FEATURE_OFFSET + i];
	}
}


//-----------------------------------------------------------
void TreePath::GetPathAverageFeatures (FeatureVector& _rFeatureVector)
{
	for (int i = AVERAGE_FEATURE_OFFSET; 
		i < AVERAGE_FEATURE_OFFSET + BASE_FEATURE_COUNT; 
		++ i) 
	{
		if (arr_PathFeatureCounts [i] > 0)
			_rFeatureVector.arr_Features [i] = arr_PathFeatureTotals [i] 
											/ (double) arr_PathFeatureCounts [i];
	}
}


//-----------------------------------------------------------
bool TreePath::HasDuplicateTitle (String& _rContentWords)
{
	bool bTitleIsDuplicate = (hmp_ContentWords.find (_rContentWords) != hmp_ContentWords.end ());
	return ((true == b_HasDuplicateTitle) || (true == bTitleIsDuplicate));
}


//-----------------------------------------------------------
ostream& operator<< (ostream& _ofs, const TreePath& _rPath)
{
	_ofs << _rPath.d_Score << "   ";
	ITERATE_COLLECTION (SectionTitle_dq_t, ((TreePath&)_rPath).dq_SectionTitles, iteStore) 
	{
		_ofs << " | ";
		ITERATE_COLLECTION (Word_dq_t, (*iteStore)->dq_Words, iteWord) 
		{
			Word* pWord = *iteWord;
			if (false == pWord->b_StartWord)
				_ofs << *pWord;
		}
	}
	return _ofs;
}


//-----------------------------------------------------------
TreePathStore::TreePathStore (TreePath* _pTocPath, 
							  Path* _pSectionPath, 
							  FeatureVector* _pFeatureVector, 
							  double _dScore)
{
	p_TocPath = _pTocPath;
	p_SectionPath = _pSectionPath;
	p_FeatureVector = _pFeatureVector;
	d_Score = _dScore;
}


//-----------------------------------------------------------
TreePathStore::~TreePathStore (void)
{
	if (NULL != p_FeatureVector)
		delete p_FeatureVector;
}


//-----------------------------------------------------------
TreeBeam::TreeBeam (bool _bTrain)
{
	i_MaxBeamWidth = _bTrain ? Config::Get ("TOC_TRAIN_BEAM_SIZE") : Config::Get ("TOC_DECODE_BEAM_SIZE");;
	i_BeamWidthToPrint = Config::Get ("TOC_BEAM_WIDTH_TO_PRINT");

	pp_Paths = new TreePath* [i_MaxBeamWidth];
	memset (pp_Paths, 0, i_MaxBeamWidth * sizeof (TreePath*));

	i_BeamWidth = 1;
	pp_Paths [0] = new TreePath;
}


//-----------------------------------------------------------
TreeBeam::~TreeBeam (void)
{
	for (int i = 0; i < i_BeamWidth; ++ i)
		delete pp_Paths [i];

	i_BeamWidth = -1;
	i_MaxBeamWidth = -2;
	delete [] pp_Paths;
}


//-----------------------------------------------------------
TreePath* TreeBeam::operator [] (int _iIndex)
{
	assert (_iIndex < i_BeamWidth);
	return pp_Paths [_iIndex];
}


//-----------------------------------------------------------
void TreeBeam::AddPath (TreePath* _pTocPath, Path* _pSectionPath, FeatureVector* _pFeatureVector, double _dScore)
{
	if ((map_AllNewPaths.size () >= (unsigned int) i_MaxBeamWidth) && (_dScore < d_CurrentMin))
	{
		delete _pFeatureVector;
	 	return;
	}

	map_AllNewPaths.insert (make_pair (_dScore, new TreePathStore (_pTocPath, _pSectionPath, _pFeatureVector, _dScore)));
	if (map_AllNewPaths.size () > (unsigned int) i_MaxBeamWidth)
	{
		Score_to_TreePathWord_map_t::iterator	ite = map_AllNewPaths.begin ();
		delete ite->second;
		map_AllNewPaths.erase (ite);
	}

	Score_to_TreePathWord_map_t::iterator ite = map_AllNewPaths.begin ();
	d_CurrentMin = ite->first;
}


//-----------------------------------------------------------
void TreeBeam::Trim (void)
{
	TreePath_map_t	mapPaths;
	for (int i = 0; i < i_BeamWidth; ++ i)
		mapPaths.insert (make_pair (pp_Paths [i], pp_Paths [i]));

	int iPathCount = 0;
	Score_to_TreePathWord_map_t::reverse_iterator	ite;
	Score_to_TreePathWord_map_t::reverse_iterator	ite_end;
	for (ite = map_AllNewPaths.rbegin (),
		ite_end = map_AllNewPaths.rend ();
		ite != ite_end;
		++ ite)
	{
		TreePathStore* pStore = ite->second;
		TreePath* pTocPath = pStore->p_TocPath;

		TreePath_map_t::iterator itePath;
		itePath = mapPaths.find (pTocPath);
		if (itePath != mapPaths.end ())
		{
			pTocPath = itePath->second;
			itePath->second = new TreePath (*pTocPath);
		}

		pTocPath->Grow (pStore->p_SectionPath, pStore->d_Score, pStore->p_FeatureVector);
		pTocPath->AccumulatePathFeatures ();
		pStore->p_FeatureVector = NULL;
		pp_Paths [iPathCount ++] = pTocPath;

		if (iPathCount >= i_MaxBeamWidth)
			break;
	}

	// delete all TreePathStore objects...
	for (ite = map_AllNewPaths.rbegin (),
		ite_end = map_AllNewPaths.rend ();
		ite != ite_end;
		++ ite)
		delete ite->second;
	map_AllNewPaths.clear ();

	i_BeamWidth = iPathCount;

	TreePath_map_t::iterator	itePath;
	for (itePath = mapPaths.begin ();
		itePath != mapPaths.end ();
		++ itePath)
		delete itePath->second;
	mapPaths.clear ();
}


//-----------------------------------------------------------
bool TreeBeam::HasYGood (void)
{
	for (int i = 0; i < i_BeamWidth; ++ i)
	{
		if ((*(pp_Paths [i])).IsYGood ())
			return true;
	}
	return false;
}


//-----------------------------------------------------------
void TreeBeam::Reset (TreePath& _rYGood)
{
	for (int i = 0; i < i_BeamWidth; ++ i)
		delete pp_Paths [i];

	pp_Paths [0] = new TreePath (_rYGood);
	i_BeamWidth = 1;
}


//-----------------------------------------------------------
void TreeBeam::StartNewSections (Section* _pSection)
{
	for (int i = 0; i < i_BeamWidth; ++ i)
		pp_Paths [i]->StartNewSection (_pSection);
}


//-----------------------------------------------------------
ostream& operator<< (ostream& _ofs, const TreeBeam& _rBeam)
{
	int iPrintWidth = _rBeam.i_BeamWidthToPrint;
	if (iPrintWidth > _rBeam.i_BeamWidth)
		iPrintWidth	= _rBeam.i_BeamWidth;

	for (int i = 0; i < iPrintWidth; ++ i) 
	{
		TreePath* pPath = ((TreeBeam&) _rBeam) [i];
		cout << i << " Section (" << pPath->dq_SectionTitles.size ()
			 << ")  " << *pPath << endl;
	}
	return _ofs;
}


//-----------------------------------------------------------
TreeDecoder::TreeDecoder (Model& _rModel)
{
	i_Order = Config::Get ("LM_ORDER");
	TreePath::p_StartWord = Word::GetStartWord ();

	p_Model = &_rModel;
	p_EndWord = Word::GetEndWord ();
}


//-----------------------------------------------------------
TreeDecoder::~TreeDecoder (void)
{
    ITERATE_COLLECTION (SectionToBeamYGood_hmp_t, hmp_SectionToBeamYGood, ite) 
	{
		pair <Beam*, Path*> pairBeamPath = ite->second;
		delete pairBeamPath.first;
		delete pairBeamPath.second;
    }
}


//-----------------------------------------------------------
void TreeDecoder::ExtractAverageFeatures (FeatureVector& _fvToc, 
										  FeatureVector& _fvSection)
{
	for (int i = 0; i < BASE_FEATURE_COUNT; ++ i)
		_fvToc.arr_Features [i] = _fvSection.arr_Features [i + AVERAGE_FEATURE_OFFSET];
	_fvToc.arr_Features [ETLM] = _fvSection.arr_Features [ETLM];
}

//-----------------------------------------------------------
int TreeDecoder::GetYGood (Beam &_beam, Path* pYGood) 
{
	int iYGoodIndex = -1;
	int iYGoodSimilarity = -1;
    
	for (int i = 0; i < _beam.i_BeamWidth; ++i) 
	{
		register int iSimilarity = pYGood->GetSimilarity (_beam [i]);
		if (iSimilarity > iYGoodSimilarity) 
		{
			iYGoodSimilarity = iSimilarity;
			iYGoodIndex = i;
		}
	}
    
    return iYGoodIndex;
}

//-----------------------------------------------------------
Section_to_Title_map_t TreeDecoder::BeamSearch (Section* _pSection, bool _bTrain)
{
	Perceptron& rPerceptron = p_Model->o_TocPerceptron;

	bool bShowProgress = (1 == (int) Config::Get ("TOC_SHOW_PROGRESS"));
	bool bUseSectionAverageFeatures = (1 == (int) Config::Get ("USE_SECTION_AVERAGE_FEATURES"));
	String sWeightsFileName (Config::Get ("TOC_WEIGHTS_FILE_PATH"));
	int iWindowSize = Config::Get ("WINDOW_SIZE");
	TreePath::i_InitialStartWords = iWindowSize - 1;
	SectionTitle::i_TagNGramSize = Config::Get ("FEATURE_TOC_TAG_NGRAM_SIZE");

	TreeBeam beam (_bTrain);
	TreePath oYGood;
	oYGood.p_FeatureVector = new FeatureVector;

	String sFeatureVectorFilePath = Config::Get ("WEIGHTS_FILE_PATH");

	Section_list_t lstSections;
	if (true == b_HierarchicalDecoding)
		Utils::GetInOrderSectionList (lstSections, _pSection);
	else
		lstSections.push_back (_pSection);

	Section* pLastSection = lstSections.back ();
	ITERATE_COLLECTION (Section_list_t, lstSections, iteSection) 
	{
		Section* pSection = *iteSection;

		oYGood.StartNewSection (pSection);
		beam.StartNewSections (pSection);

		int iPathLength = Config::Get ("TITLE_LENGTH");
		if (true == _bTrain)
			iPathLength = pSection->p_Title->lst_Words.size ();
		else
		{
			if (true == pSection->p_Title->lst_Words.empty ())
				continue;
			if (0 == iPathLength)
				iPathLength = Utils::GetTitleWordLength (pSection->p_Title->s_Text) - 1;
		}
		if (iPathLength <= 0)
			continue;
		if (true == pSection->dq_UniqueWords.empty ())
			continue;


		// Get/Cache decoder output
		Beam* pSectionBeam;
		Path* pSectionYGood;

		SectionToBeamYGood_hmp_t::iterator iteSectionBeamYGood;
		iteSectionBeamYGood = hmp_SectionToBeamYGood.find ((long) pSection);
		if (iteSectionBeamYGood == hmp_SectionToBeamYGood.end ())
		{
		    Decoder decoder (*p_Model, DECODE);
		    decoder.BeamSearch (pSection, DECODE, TOC);

			if (true == _bTrain)
				pSectionBeam = new Beam (decoder.beam, true);
			else
			{
				int iBeamWidth = decoder.beam.i_BeamWidth;
				if (iBeamWidth > 5)
					decoder.beam.i_BeamWidth = 5;
				pSectionBeam = new Beam (decoder.beam, true);
				decoder.beam.i_BeamWidth = iBeamWidth;
			}

		    for (int i = 0; i < pSectionBeam->i_BeamWidth; ++ i)
				(*pSectionBeam) [i]->b_YGoodMatch = false;

		    int iYBestIndex = GetYGood (*pSectionBeam, decoder.p_YGood);
		    (*pSectionBeam) [iYBestIndex]->b_YGoodMatch = true;
		    pSectionYGood = new Path (*(*pSectionBeam) [iYBestIndex]); 
		    cout << "YBest: " << *pSectionYGood << endl;

		    hmp_SectionToBeamYGood.insert (make_pair ((long) pSection, make_pair (pSectionBeam, pSectionYGood)));
		}
		else
		{
		    pair <Beam*, Path*> pairBeamPath = iteSectionBeamYGood->second;
		    pSectionBeam = pairBeamPath.first;
		    pSectionYGood = pairBeamPath.second;

			// FeatureVector::Init is normally called in Decoder::BeamSearch.	
			// When we retrieve the Decoder beam from the cache without calling	
			// BeamSearch, we need to set it ourselves...						
			FeatureVector::Init (*p_Model, pSection);
		}
		
		// grow the current path beam using the section beams...
		for (int iSectionPath = 0; iSectionPath < pSectionBeam->i_BeamWidth; ++ iSectionPath)
		{
			Path* pSectionPath = (*pSectionBeam) [iSectionPath];
			SectionTitle oSectionTitle (pSection, pSectionPath);

			for (int iTocPath = 0; iTocPath < beam.i_BeamWidth; ++ iTocPath)
			{
				TreePath* pTocPath = beam [iTocPath];

				FeatureVector* pFV = new FeatureVector;
				if (true == bUseSectionAverageFeatures)
					ExtractAverageFeatures (*pFV, *pSectionPath->p_FeatureVector);
				pFV->CreateTreeFeatures (pTocPath, 
										pSectionPath, 
										&oSectionTitle, 
										iSectionPath, 
										pSectionBeam->i_BeamWidth);
				pTocPath->GetPathAverageFeatures (*pFV);

				double dScore = rPerceptron.Evaluate (*pFV);

				beam.AddPath (pTocPath, pSectionPath, pFV, dScore);
			}
		}

		beam.Trim ();
		if (true == bShowProgress)
			if (false == _bTrain)
				cout << "   " << beam << endl;

		if (true == _bTrain) 
		{
			// find the location of YGood in the section beam...
			int iYGoodIndex = 0;
			for (; iYGoodIndex < pSectionBeam->i_BeamWidth; ++ iYGoodIndex)
			{
				if (true == (*pSectionBeam) [iYGoodIndex]->b_YGoodMatch)
					break;
			}
			SectionTitle oYGoodTitle (pSection, pSectionYGood);
			oYGood.p_FeatureVector->Clear ();
			if (true == bUseSectionAverageFeatures)
				ExtractAverageFeatures (*oYGood.p_FeatureVector, *pSectionYGood->p_FeatureVector);
			oYGood.p_FeatureVector->CreateTreeFeatures (&oYGood, 
														pSectionYGood, 
														&oYGoodTitle, 
														iYGoodIndex, 
														pSectionBeam->i_BeamWidth);
			oYGood.GetPathAverageFeatures (*oYGood.p_FeatureVector);
			oYGood.Grow (pSectionYGood, 0, oYGood.p_FeatureVector);
			oYGood.AccumulatePathFeatures ();


			// save feature weights to file for debugging...											
			if (false)
			{
				String sYGoodFile;
				sYGoodFile << sFeatureVectorFilePath << ".yg";
				String sYBadFile;
				sYBadFile << sFeatureVectorFilePath << ".yb";
				oYGood.p_FeatureVector->Save (sYGoodFile, oYGood.p_LastSectionTitle->s_Title, true);
				if (false == beam [0]->IsYGood ())
					beam [0]->p_FeatureVector->Save (sYBadFile, beam [0]->p_LastSectionTitle->s_Title, true);
				else if (beam.i_BeamWidth > 1)
					beam [1]->p_FeatureVector->Save (sYBadFile, beam [1]->p_LastSectionTitle->s_Title, true);
			}

			// last update :																			
			//   If this is the last section, we have finished generating the TOC, so the hypothesis	
			//   at the top of the beam has to be YGood.												
			//   Is this is not the last section, we're still in the middle of generating the TOC, so	
			//   it's good enough for the YGood to be somewhere in the beam...							
			bool bUpdateNotRequired = (pSection == pLastSection)?  beam [0]->IsYGood () : beam.HasYGood ();
			if (false == bUpdateNotRequired)
			{
				if (true == bShowProgress)
				{
					if (pSection == pLastSection)
						cout << endl << "     == Y Good : " << oYGood << endl << "     " << beam << endl;
					else
						cout << endl << "     ## Y Good : " << oYGood << endl << "     " << beam << endl;
				}

				Feature_Vector_list_t lstFVBad;
				for (int iBeam = 0; iBeam < beam.i_BeamWidth; ++ iBeam)
				{
					TreePath* pPath = beam [iBeam];
					if ((pSection == pLastSection) && (pPath->IsYGood ()))
						break;
					lstFVBad.push_back (pPath->p_FeatureVector);
				}

				rPerceptron.Update (*oYGood.p_FeatureVector, lstFVBad);
				beam.Reset (oYGood);
			}
			else if (true == bShowProgress)
			{
				cout << '~';

				// debug >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
				int iStartingBeamWidth = beam.i_BeamWidth;
				int iBadCount = 0;
				for (int i = 0; i < beam.i_BeamWidth; ++ i)
				{
					if (false == beam [i]->IsYGood ())
						++ iBadCount;
				}
				if (iBadCount > 0)
					cout << iBadCount << '/' << iStartingBeamWidth;
				// debug <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
			
				cout.flush ();
			}
		}
	}

	// when we get here, beam [0] is the highest scoring path...	
	Section_to_Title_map_t mapSectionToTitle;
	if (true == _bTrain)
	{
		if (true == bShowProgress)
			cout << endl;
	}
	else
	{
		if (true == bShowProgress)
		{
			cout << "-----------------------------------------------" << endl;
			cout << beam << endl;
		}

		// construct title structure used for printing out the tree of titles
		ITERATE_COLLECTION (SectionToTitle_hmp_t, beam [0]->hmp_SectionToTitle, iteTitleStore)
		{
			SectionTitle* pTitle = iteTitleStore->second;
			mapSectionToTitle.insert (make_pair (iteTitleStore->first, pTitle->s_Title));
		}
	}

	return mapSectionToTitle;
}


