#ifndef __TITLE_GEN_TOC_DECODER__
#define __TITLE_GEN_TOC_DECODER__

#include <set>
#include <list>
#include <deque>
#include <iostream>
#include "Section.h"
#include "FeatureVector.h"
#include "Perceptron.h"
#include "Model.h"
#include "hash_map.hpp"

#define AVG_PREFIX 			'*'
#define SECTION_AVG_PREFIX	"s_"
#define PATH_AVG_PREFIX		"p_"

#define MIDDLE_OF_TITLE		false
#define END_OF_TITLE		true


//-----------------------------------------------------------
typedef deque <Word*>				Word_dq_t;
typedef set   <String>				Unigram_set_t;
typedef hash_map <long, String>		Section_to_Title_map_t;

class Beam;
class Path;



//-----------------------------------------------------------
class SectionTitle
{
	public:
		static int				i_TagNGramSize;
		Unigram_set_t			set_TitleStems;
		Unigram_to_Count_map_t	map_StemCounts;
		Unigram_to_Count_map_t	map_TagNgramCounts;
		Word_dq_t				dq_Words;
		Section*				p_Section;
		String					s_Title;
		String					s_TitleStems;
		String					s_ContentWords;


		SectionTitle (SectionTitle& _rOld);
		SectionTitle (Section* p_Section, Path* _pSectionPath);
};



//-----------------------------------------------------------
typedef deque <SectionTitle*>						SectionTitle_dq_t;
typedef list <Word_dq_t*>							SectionTitles_lst_t;
typedef list <Word*>								Word_list_t;
typedef hash_map <String, bool>						WordTag_hash_t;
typedef hash_map <long, SectionTitle*>				SectionToTitle_hmp_t;
typedef hash_map <long, SectionTitle_dq_t>			SectionToTitleDQ_hmp_t;
typedef hash_map <String, bool>						TitleStem_hmp_t;
typedef hash_map <String, bool>						ContentWords_hmp_t;




//-----------------------------------------------------------
class TreePath
{
	public:
		static Word*				p_StartWord;
		static int					i_InitialStartWords;

		SectionToTitleDQ_hmp_t		hmp_SectionToTitleStoreDQ;
		SectionToTitle_hmp_t		hmp_SectionToTitle;
		SectionTitle_dq_t			dq_SectionTitles;
		TitleStem_hmp_t				hmp_TitleStems;
		ContentWords_hmp_t			hmp_ContentWords;

		Section*					p_CurrentSection;
		SectionTitle*				p_LastSectionTitle;
		FeatureVector*				p_FeatureVector;

		double						arr_PathFeatureTotals [FEATURE_COUNT];
		long						arr_PathFeatureCounts [FEATURE_COUNT];

		double						d_SectionPathScoreTotal;
		double						d_Score;
		bool						b_YGoodMatch;
		bool						b_LastWordIsNotInSection;		
		bool						b_HasDuplicateTitle;
		
		TreePath (void);
		TreePath (TreePath& _rOld);
		~TreePath (void);
		void StartNewSection (Section* _pSection);
		void Grow (Path* _pSectionpath,
				   double _dScore, 
				   FeatureVector* _pFeatureVector);

		SectionTitle* GetSectionTitle (Section* _pSection);
		SectionTitle_dq_t* GetPreviousSiblings (Section* _pSection);

		void AccumulatePathFeatures (void);
		void GetPathAverageFeatures (FeatureVector& _rFeatureVector);
		void GetSectionAverageFeatures (FeatureVector& _rFeatureVector);

		bool HasWordInLastTitle (Word& _rWord);
		bool IsYGood (void);
		bool HasDuplicateTitle (String& _rContentWords);
		bool operator< (TreePath& _rRight);
};

ostream& operator<< (ostream& _ofs, const TreePath& _rPath);



//-----------------------------------------------------------
class TreePath;
class TreePathStore;
class TreeBeam;
class TreeDecoder;
typedef multimap <TreePath*, TreePath*>			TreePath_map_t;
typedef multimap <double, TreePathStore*>		Score_to_TreePathWord_map_t;



//-----------------------------------------------------------
class TreePathStore
{
	public:
		TreePath*		p_TocPath;
		Path*			p_SectionPath;
		FeatureVector*	p_FeatureVector;
		double			d_Score;

		TreePathStore (TreePath* _pTocPath, Path* _pSectionPath, 
						FeatureVector* _pFeatureVector, double _dScore);
		~TreePathStore (void);
};



//-----------------------------------------------------------
class TreeBeam
{
	private:
		TreePath**					pp_Paths;
		double						d_CurrentMin;
		
	public:
		Score_to_TreePathWord_map_t	map_AllNewPaths;
		int							i_MaxBeamWidth;
		int							i_BeamWidth;
		int							i_BeamWidthToPrint;
		
		TreeBeam (bool _bTrain);
		~TreeBeam (void);

		TreePath* operator[] (int _iIndex);
		void AddPath (TreePath* _pTocPath, Path* _pSectionPath, FeatureVector* _pFeatureVector, double _dScore);
		void Trim (void);
		void GetFeatureVectors (Feature_Vector_list_t &_lstFeatureVectors );

		bool HasYGood (void);
		void Reset (TreePath& _rYGood);
		void StartNewSections (Section* _pSection );
};

ostream& operator<< (ostream& _ofs, const TreeBeam& _rBeam);


//-----------------------------------------------------------
typedef hash_map <long, pair< Beam*, Path* > >                  SectionToBeamYGood_hmp_t;



//-----------------------------------------------------------
class TreeDecoder
{
	private:
		static bool					b_Complete;
		static bool					b_LastWordEvaluation;
		static Word*				p_EndWord;

		pthread_t* 					p_Thread;
		int							i_ThreadCount;
		SectionToBeamYGood_hmp_t	hmp_SectionToBeamYGood;
		Model*						p_Model;


		int GetYGood( Beam &_beam, Path* pYGood );
		void ExtractAverageFeatures (FeatureVector& _fvToc, FeatureVector& _fvSection);

	public:
		static int i_Order;
		static bool b_HierarchicalDecoding;

		TreeDecoder (Model& _rModel);
		~TreeDecoder (void);
		
		Section_to_Title_map_t BeamSearch (Section* _pSection, bool _bTrain);
};



#endif
