#include "FeatureVector.h"
#include "math.h"
#include "Utils.h"
#include "Section.h"
#include "Config.h"
#include "TreeDecoder.h"
#include "Decoder.h"
#include <math.h>

using namespace std;


bool FeatureVector::b_UseHierarchicalFeatures = true;


//-------------------------------------------------------------------
void FeatureVector::CreateTreeFeatures (TreePath* _pTocPath, 
										Path* _pSectionPath, 
										SectionTitle* _pSectionTitle,
										int _iSectionBeamIndex,
										int _iBeamWidth)
{
	// local re-ranking features													  
	if (false == _pSectionPath->dq_Words.empty ())
	{
		Word* pLastWord = _pSectionPath->dq_Words [_pSectionPath->dq_Words.size () - 1];
		if ((pLastWord->s_POSTag == "JJ") || 
			(pLastWord->s_POSTag == "JJR") ||
			(pLastWord->s_POSTag == "JJS"))
			AddFeature (HR_JJE, 1);
	}
	
	// the return in the line below is to stop the hierarchical features from firing !
	if (false == b_UseHierarchicalFeatures)
		return;

	// Averaged section decoder path score as a feature....
	AddFeature (H_SPS, (_pTocPath->d_SectionPathScoreTotal + _pSectionPath->d_Score) 
	 					/ (double) (_pTocPath->dq_SectionTitles.size () + 1));
	AddFeature (H_SPSL, _pSectionPath->d_Score);

	// REDUNDANCY FEATURES //////////////////////////////
	int iTitleDuplicateByStem = (_pTocPath->HasDuplicateTitle (_pSectionTitle->s_ContentWords))? 1 : 0;
	AddFeature (H_DT, iTitleDuplicateByStem); 

	Section* pParent = p_Section->p_Parent;
	SectionTitle* pParentTitleStore = (pParent != NULL) ? _pTocPath->GetSectionTitle (pParent) : NULL;
	SectionTitle_dq_t* pdqPrevSiblingStores = _pTocPath->GetPreviousSiblings (pParent);
	Unigram_set_t* pTitleStems =  &_pSectionTitle->set_TitleStems;
	int iTotalUniqueStems = pTitleStems->size ();
	bool bTotalUniqueStemsNonZero = (iTotalUniqueStems > 0);
	long lDQStoreSize = _pSectionTitle->dq_Words.size ();
	int iTotalStems = lDQStoreSize - TreeDecoder::i_Order + 1;
	bool bTotalStemsNonZero = (iTotalStems > 0);

	bool bParentTitleNonEmpty = false;
	if (NULL != pParentTitleStore)
	    bParentTitleNonEmpty = (false == pParentTitleStore->dq_Words.empty ());

	// adding garbage word or checking for empty titles					
	long lFirstWordIndex = TreeDecoder::i_Order - 1;
	Word* pFirstWord = _pSectionTitle->dq_Words [lFirstWordIndex];
	Word* pLastWord = _pSectionTitle->dq_Words [lDQStoreSize - 1];
	
	int iTotalSiblings = 0;
	if (NULL != pdqPrevSiblingStores)
		iTotalSiblings = pdqPrevSiblingStores->size () - 1;

	// Previous section features
	Section* pPrevious = p_Section->p_PreviousSection;
	SectionTitle* pPrevTitleStore = ( pPrevious != NULL ) ? _pTocPath->GetSectionTitle( pPrevious ) : NULL;	
	if( pPrevTitleStore != NULL ) {
	    
	    // Same title as previous section
	    if (pPrevTitleStore->s_ContentWords == _pSectionTitle->s_ContentWords)
			AddFeature (HPRE_ST, 1.0);
	    // Not same title as previous but ends with the same word
	    else
		{
			/*
		if( ( pPrevious->p_Parent != NULL ) && ( _pTocPath->GetSectionTitle( pPrevious->p_Parent ) == pParentTitleStore ) ){
		    if ( ( lDQStoreSize >  2 )  && 
			 ( *( pPrevTitleStore->dq_Words [pPrevTitleStore->dq_Words.size () - 1] ) == *pLastWord ) )
			AddFeature (HS_SLW, 1.0);
		    if ( ( lDQStoreSize >  2 )  && 
			 ( *( pPrevTitleStore->dq_Words [ TreeDecoder::i_Order - 1 ] ) == *pFirstWord ) )
			AddFeature (HS_SFW, 1.0);
		*/
		
		// Number of same stems as previous, normalized		
		//if (true == bTotalStemsNonZero) {
		//	int iPreviousMatches = Utils::SetIntersectionSize (*pTitleStems, pPrevTitleStore->set_TitleStems); 
		//	AddFeature (HPRE_SS, iPreviousMatches / (double) iTotalStems);
		//  }
		
			Section* pPrevious2 = pPrevious->p_PreviousSection;
			SectionTitle* pPrevTitleStore2 = (NULL != pPrevious2)? _pTocPath->GetSectionTitle (pPrevious2) : NULL;
			if (NULL != pPrevTitleStore2)
		    {
				// Same title as previous section
				if (pPrevTitleStore2->s_ContentWords == _pSectionTitle->s_ContentWords)
					AddFeature (HPRE2_ST, 1.0);
		    }
	    }
	}
	    
	// Same title as cousin which is not in the same position
	/*
	if( pParent != NULL ) {
	    Section* pGrandParent =  ( pParent == NULL ) ? NULL :  pParent->p_Parent;
	    if( pGrandParent != NULL ) {
		ITERATE_COLLECTION( Section_list_t, pGrandParent->lst_Children, iteUncle ) {
		    bool bFeatureAdded = false;
		    Section * pUncle = *iteUncle;
		    if( pUncle == pParent) 
			break;
		    if ( _pTocPath->GetSectionTitle( pUncle ) == NULL )
			continue;
		    
		    register int iCousinIndex = 0;
		    ITERATE_COLLECTION( Section_list_t, pUncle->lst_Children, iteCousin ) {

			if( iCousinIndex == iTotalSiblings ) 
			    continue;

			Section *pCousin = *iteCousin;
			SectionTitle* pCousinTitleStore = _pTocPath->GetSectionTitle( pCousin );
			if ( ( pCousinTitleStore != NULL ) &&
			     ( pCousinTitleStore->s_ContentWords == _pSectionTitle->s_ContentWords ) ) {
			    AddFeature ( HC_ST, 1.0 );
			    bFeatureAdded = true;
			    break; 
			}
		    }

		    if( true == bFeatureAdded ) 
			break;
		}
	    }
	}
	*/
	
	// Parent section features
	{
		// cosine similarity structures...									
		Unigram_to_Count_map_t* pStemCounts = &_pSectionTitle->map_StemCounts;
		int iTotalStemsSquared = iTotalStems * iTotalStems;

		// Number of POS tag trigram matches								
		Unigram_to_Count_map_t* pTagNgramCounts = &_pSectionTitle->map_TagNgramCounts;	
		int iTotalMatches = pTagNgramCounts->size ();
       
		if ((NULL != pParent) && (NULL != pParentTitleStore) && (true == bParentTitleNonEmpty))
		{
			// long lParentTitleLength = pParentTitleStore->dq_Words.size ();

			// Same title as parent								
			if (pParentTitleStore->s_ContentWords == _pSectionTitle->s_ContentWords)
				AddFeature (HP_ST, 1.0);


			// Number of same stems as parent, normalized		
			if (true == bTotalStemsNonZero)
			{
				int iParentMatches = Utils::SetIntersectionSize (*pTitleStems, pParentTitleStore->set_TitleStems); 
				AddFeature (HP_SS, iParentMatches / (double) iTotalStems);
			}

			// Parent section starts with same first word		
			//if (lFirstWordIndex < lParentTitleLength)
			//	if (*pParentTitleStore->dq_Words [lFirstWordIndex] == *pFirstWord)
			//		AddFeature (HP_SFW, 1.0);
		
			// Parent section ends with same last word			
			//if (* (pParentTitleStore->dq_Words [pParentTitleStore->dq_Words.size () - 1]) == *pLastWord)
			//	AddFeature (HP_SLW, 1.0);

			// cosine similarity of title to parent title		
			int iTotalParentStems = (pParentTitleStore->dq_Words.size () - TreeDecoder::i_Order);
			if ((0 != iTotalStems) || (0 != iTotalParentStems))
			{
				double dParentCosSim = 0;
				ITERATE_COLLECTION_POINTER (Unigram_to_Count_map_t, pStemCounts, iteStemCount) 
				{
					Unigram_to_Count_map_t::iterator iteParentStemCount;
					iteParentStemCount = pParentTitleStore->map_StemCounts.find (iteStemCount->first);
					if (iteParentStemCount != pParentTitleStore->map_StemCounts.end ())
						dParentCosSim += iteStemCount->second;
				}
				dParentCosSim /= sqrt ((double) iTotalStemsSquared + (iTotalParentStems * iTotalParentStems));
				AddFeature (HP_CS, dParentCosSim);
			}

			// Number of POS tag trigram matches								
			Unigram_to_Count_map_t* pParentTagNgramCounts = &pParentTitleStore->map_TagNgramCounts;
			int iParentMatches = 0;
			ITERATE_COLLECTION_POINTER (Unigram_to_Count_map_t, pTagNgramCounts, iteNgram) 
			{
				if (pParentTagNgramCounts->find (iteNgram->first) != pParentTagNgramCounts->end ())
					++ iParentMatches;
			}
			if (iTotalMatches > 0)
				AddFeature (HP_TM, iParentMatches / (double) iTotalMatches);
		}

		if (iTotalSiblings > 0) 
		{
			// Number of siblings with same title							
			int iSiblingSameTitle = 0;
			// Number of same stems as siblings (average and max)			
			int iMaxSiblingMatches = 0;
			int iTotalSiblingMatches = 0;
			// Sibling sections that start/end with same first and last word
			int iSiblingFirstWordMatches = 0;
			int iSiblingLastWordMatches = 0;
			// Cosine Similarity with siblings								
			double dMaxSiblingCosSim = 0;
			// Number of POS tag trigram matches								
			int iSiblingMatches = 0;

			// calculations													
			typedef deque <SectionTitle*> dqSectionTitleStore_t;
			ITERATE_COLLECTION_POINTER (dqSectionTitleStore_t, pdqPrevSiblingStores, iteSiblingStore) 
			{
				SectionTitle* pSiblingStore = *iteSiblingStore;

				// we don't want to compare the section with itself! so skip over it...
				if (pSiblingStore == _pSectionTitle)
					continue;
				// no need to calculate values if the sibling's title is empty !		
				if (true == pSiblingStore->dq_Words.empty ())
					continue;
				long lSiblingTitleLength = pSiblingStore->dq_Words.size ();

				if (pSiblingStore->s_ContentWords == _pSectionTitle->s_ContentWords)
					++ iSiblingSameTitle;

				if (true == bTotalStemsNonZero)
				{
					register int iSiblingMatches = Utils::SetIntersectionSize (*pTitleStems, 
												   pSiblingStore->set_TitleStems);
					// iMaxSiblingMatches = iMaxSiblingMatches <? iSiblingMatches;
					if (iMaxSiblingMatches < iSiblingMatches)
						iMaxSiblingMatches = iSiblingMatches;
					iTotalSiblingMatches += iSiblingMatches;
				}

				if (lFirstWordIndex < lSiblingTitleLength)
					if (* (pSiblingStore->dq_Words [lFirstWordIndex]) == *pFirstWord)
						++ iSiblingFirstWordMatches;
				if ( *(pSiblingStore->dq_Words [pSiblingStore->dq_Words.size () - 1]) == *pLastWord )
				    ++ iSiblingLastWordMatches;
				

				// cosine similarity calculation							
				// TODO: Ignore stopwords ?
				double dSiblingCosSim = 0.0;
				ITERATE_COLLECTION_POINTER (Unigram_to_Count_map_t, pStemCounts, iteStemCount) 
				{
					Unigram_to_Count_map_t::iterator iteSiblingStemCount;
					iteSiblingStemCount = pSiblingStore->map_StemCounts.find (iteStemCount->first);
					if (iteSiblingStemCount != pSiblingStore->map_StemCounts.end ())
						dSiblingCosSim += iteStemCount->second;
				}
				int iTotalSiblingStems = (pSiblingStore->dq_Words.size () - TreeDecoder::i_Order);
				dSiblingCosSim /= sqrt ((double) iTotalStemsSquared + (iTotalSiblingStems * iTotalSiblingStems));
				// dMaxSiblingCosSim = dMaxSiblingCosSim <? dSiblingCosSim;
				if (dMaxSiblingCosSim < dSiblingCosSim)
					dMaxSiblingCosSim = dSiblingCosSim;

				// Number of POS tag trigram matches						
				Unigram_to_Count_map_t& rSiblingTagNgramCounts = pSiblingStore->map_TagNgramCounts;
				ITERATE_COLLECTION_POINTER (Unigram_to_Count_map_t, pTagNgramCounts, iteNgram) 
				{
					if (rSiblingTagNgramCounts.find (iteNgram->first) != rSiblingTagNgramCounts.end ())
						++ iSiblingMatches;
				}
			}

			//if (iSiblingSameTitle > 0)
			//	AddFeature (HS_ST, 1);

			if (true == bTotalUniqueStemsNonZero)
			{
				AddFeature (HS_SSM, 1.0 * iMaxSiblingMatches / iTotalUniqueStems);
				AddFeature (HS_SSA, (1.0 * iTotalSiblingMatches / iTotalSiblings) / iTotalUniqueStems);
			}

			// Feature:S tarts/Ends with same word						
			Section* pRoot = p_Section->GetRoot ();
			double dTotalUnigramOccurrences = p_Section->GetTotalUnigramOccurrences ();
			double dFirstWordTF = p_Section->GetUnigramOccurrence (pFirstWord->s_Stem, true) 
									/ dTotalUnigramOccurrences;
			double dFirstWordDF = pRoot->GetNumberOfSectionsWithUnigram (pFirstWord->s_Stem, true) 
									/ (double) i_SectionCount;
			double dFirstWordTFIDF = - dFirstWordTF / log (dFirstWordDF);

			double dLastWordTF = p_Section->GetUnigramOccurrence (pLastWord->s_Stem, true) 
									/ dTotalUnigramOccurrences;
			double dLastWordDF = pRoot->GetNumberOfSectionsWithUnigram (pLastWord->s_Stem, true) 
									/ (double) i_SectionCount;
			double dLastWordTFIDF = - dLastWordTF / log (dLastWordDF);

			AddFeature (HS_SFW, dFirstWordTFIDF);
			AddFeature (HS_SLW, dLastWordTFIDF);
			// cosine similarity of title to sibling titles (averaged)	
			AddFeature (HS_CS, dMaxSiblingCosSim);
			// Number of POS tag trigram matches						
			if ((iTotalSiblings > 0) && (iTotalMatches > 0))
				AddFeature (HS_TMA, (1.0 * iSiblingMatches / iTotalSiblings) / iTotalMatches);
		}
	}


	// POSITION FEATURES //////////////////////////////	
	// Position relative to Analysis
	{
	}
}


