#include "Utils.h"
#include "Config.h"
#include "PorterStemmer.h"

Unigram_set_t Utils::set_NP_Tags;

//-----------------------------------------------------
// The funtionality of this method is specific to the  
// corpus used, and removes the formatting code        
// contained therein.                                  
//-----------------------------------------------------
void Utils::CleanText (String& _rText)
{
    _rText.Replace (">th ", "> ");
    _rText.Replace (" ax ", " ");
    _rText.Replace (" py ", " ");
    _rText.ReplacePattern ("\\&.*\\&", " <exp0> ");
    _rText.ReplacePattern ("\\{.*\\}", " <exp1> ");
    _rText.Replace ("<expression>", "<exp>");
    _rText.Replace ("<figure>", "<exp>");
    _rText.Replace ("<num>", "9999");
    _rText.ReplacePattern ("([A-Za-z0-9]+[\\+=%^*_]+[A-Za-z0-9]*)|(([\\+=%^*_]+[A-Za-z0-9]+)[\\+=%^*_]*)", "");
    _rText.Replace ("\\", "");
    _rText.ReplacePattern ("(\\([^a-zA-Z0-9]*\\))|(\\[[^a-zA-Z0-9]*\\])|(\\<[^a-zA-Z0-9]*\\>)", "");
    _rText.ReplacePattern ("([,.?*!@$%^&])[\t ]*([,.?*!@$%^&])", "\\2");
    _rText.ReplacePattern (">[>]+", ">");

    _rText.ReplacePattern ("[ ]+$", "");
}



//-----------------------------------------------------
void Utils::TokenizeText (String& _rText)
{
    _rText.Replace ("...", " ... ");

    _rText.ReplacePattern ("([,;:@#$%&])", " \\1 ");

    // Assume sentence tokenization has been done first, so split FINAL periods only.
    _rText.ReplacePattern ("([^.])([.])([]\\)}>\"\']*)[ \t]*$", "\\1 \\2\\3 ");

    // however, we may as well split ALL question marks and exclamation points,
    // since they shouldn't have the abbrev.-marker ambiguity problem
    _rText.Replace ("?", " ? ");
    _rText.Replace ("!", " ! ");

    // parentheses, brackets, etc.
    // _rText.ReplacePattern (" ([[ (){}])", " \\1 ");
    // _rText.Replace ("]", " ] ");

    _rText.Replace ("<", " <");
    _rText.Replace (">", "> ");
    _rText.Replace ("--", " -- ");
    _rText.Replace ("-", " ");

    // Some taggers, such as Adwait Ratnaparkhi's MXPOST, use the parsed-file version of these symbols.
    // UNCOMMENT THE FOLLOWING 6 LINES if you're using MXPOST.
    // s/ (/-LRB-/g
    _rText.Replace ("(", " -LRB- ");
    _rText.Replace (")", " -RRB- ");
    _rText.Replace ("[", " -LSB- ");
    _rText.Replace ("]", " -RSB- ");
    _rText.Replace ("{", " -LCB- ");
    _rText.Replace ("}", " -RCB- ");

    // First off, add a space to the beginning and end of each line, to reduce
    // necessary number of regexps.
    // String s (" ");
    // s << _rText << ' ';
    // _rText = s;
    _rText.Replace ("\"", " '' ");

	_rText.Replace ("/", " / ");

    // possessive or close-single-quote
    _rText.ReplacePattern ("([^'])' ", "\\1 ' ");

    // as in it's, I'm, we'd
    _rText.ReplacePattern ("'([sSmMdD])", " '\\1 ");

    _rText.Replace ("'ll ", " 'll ");
    _rText.Replace ("'LL ", " 'LL ");
    _rText.Replace ("'re ", " 're ");
    _rText.Replace ("'RE ", " 'RE ");
    _rText.Replace ("'ve ", " 've ");
    _rText.Replace ("'VE ", " 'VE ");
    _rText.Replace ("n't ", " n't ");
    _rText.Replace ("N'T ", " N'T ");

    _rText.ReplacePattern ("([Cc])annot ", " \\1an not ");
    // _rText.ReplacePattern ("([Dd])'ye ", " \\1id you ");
    // _rText.ReplacePattern ("([Gg])imme ", " \\1ive me ");
    //" \1im me ", text);
    // _rText.ReplacePattern ("([Gg])onna ", " \\1oing to ");
    //" \1on na ", text);
    // _rText.ReplacePattern ("([Gg])otta ", " \\1ot to ");
    //" \1ot ta ", text);
    // _rText.ReplacePattern ("([Ll])emme ", " \\1et me ");
    //" \1em me ", text);
    // _rText.ReplacePattern ("([Mm])ore'n ", " \\1ore than ");
    //" \1ore 'n ", text);
    // _rText.Replace (" 'Tis ", " It is ");
    // _rText.Replace (" 'tis ", " it is ");
    //_rText.Replace (" '([Tt])is ")
    //" \1 is ", text);
    // _rText.Replace (" 'Twas ", " It was ");
    // _rText.Replace (" 'twas ", " it was ");
    //_rText.Replace (" '([Tt])was ")
    //" \1 was ", text);
    // _rText.ReplacePattern ("([Ww])anna ", " \\1ant to ");
    //" \1an na ", text);

    _rText.ReplacePattern ("[ \t]+$", "");
    _rText.ReplacePattern ("^[ \t]+", "");
    _rText.ReplacePattern ("[ \t]+", " ");

	if (_rText.length () > 0)
	{
		char ch = _rText [_rText.length () - 1];
		if (NULL == strchr (".,!?;:", ch))
			_rText << " .";
	}
}



//-----------------------------------------------------
void Utils::SplitIntoWords (String& _rText, Unigram_list_t& _lstWords)
{
    _rText.Split (_lstWords);

    ITERATE_COLLECTION (Unigram_list_t, _lstWords, ite) 
	{
        if (false == ite->IsAlNum ())
		{
			ite = _lstWords.erase (ite);
			-- ite;
		}
    }
}



//-----------------------------------------------------
void Utils::SplitIntoSentences (String& _rText, list <String>& _lstSentences)
{
    String sDelimiter = Config::Get ("SENTENCE_DELIMITER");
    String sDelimiterType = Config::Get ("SENTENCE_DELIMITER_TYPE");

    if (sDelimiterType == "PATTERN")
        _rText.SplitByRegexp (_lstSentences, sDelimiter);
    else if (sDelimiterType == "STRING")
        _rText.SplitByString (_lstSentences, sDelimiter);
    else if (sDelimiterType == "LIST")
        _rText.SplitBySeparatorList (_lstSentences, sDelimiter);
}



//-----------------------------------------------------
void Utils::ExtractNounPhrases (Ngram_list_t &_lstNgramList, Word_dq_t& _rdqWords)
{
    if (Utils::set_NP_Tags.empty ())
	{
		Utils::set_NP_Tags.insert ("JJ");
		Utils::set_NP_Tags.insert ("JJS");
		Utils::set_NP_Tags.insert ("JJR");
		Utils::set_NP_Tags.insert ("NN");
		Utils::set_NP_Tags.insert ("NNS");
		Utils::set_NP_Tags.insert ("NNP");
		Utils::set_NP_Tags.insert ("NNPS");
    }
    
    Word_list_t* plstNounPhrase = new Word_list_t ();
 
    ITERATE_COLLECTION (Word_dq_t, _rdqWords, iteWord) 
	{
		Word* pWord = *iteWord;

		// If the POS tag belongs to a noun phrase
		if (Utils::set_NP_Tags.find (pWord->s_POSTag) != Utils::set_NP_Tags.end ())
			plstNounPhrase->push_back (pWord);
		else
		{
			if (plstNounPhrase->size () > 1)
			{
				_lstNgramList.push_back (plstNounPhrase);
				plstNounPhrase = new Word_list_t ();
			}
			else
				plstNounPhrase->clear ();
		}
    }

    if (plstNounPhrase->size () > 1)
		_lstNgramList.push_back (plstNounPhrase);
	else
		delete plstNounPhrase;
}



//-----------------------------------------------------
void Utils::ExtractNounPhrases (Ngram_list_t &_lstNgramList, String* _sAllWords, Word_list_t* _lstWordList)
{
    if (Utils::set_NP_Tags.empty ())
	{
		Utils::set_NP_Tags.insert ("JJ");
		Utils::set_NP_Tags.insert ("JJS");
		Utils::set_NP_Tags.insert ("JJR");
		Utils::set_NP_Tags.insert ("NN");
		Utils::set_NP_Tags.insert ("NNS");
		Utils::set_NP_Tags.insert ("NNP");
		Utils::set_NP_Tags.insert ("NNPS");
    }
    
    Unigram_list_t lstTextWords;
    _sAllWords->Split (lstTextWords);
 
    Word_list_t* plstNounPhrase = new Word_list_t ();
    Word_list_t::iterator iteSentWord = _lstWordList->begin ();
    ITERATE_COLLECTION (Unigram_list_t, lstTextWords, iteTextWord) 
	{
		if (iteSentWord == _lstWordList->end ())
			break;
	
		Word* pSentWord = *iteSentWord;
		String sTextWord = *iteTextWord;

		// If the text word matches the word object
		if (pSentWord->s_Word == sTextWord) 
		{
			// If the POS tag belongs to a noun phrase
			if (Utils::set_NP_Tags.find (pSentWord->s_POSTag) != Utils::set_NP_Tags.end ())
				plstNounPhrase->push_back (pSentWord);
			else
			{
				if (plstNounPhrase->size () > 1)
				{
					_lstNgramList.push_back (plstNounPhrase);
					plstNounPhrase = new Word_list_t ();
				}
				else
					plstNounPhrase->clear ();
			}
			++iteSentWord;

		} 
		else 
		{
			if (plstNounPhrase->size () > 1)
			{
				_lstNgramList.push_back (plstNounPhrase);
				plstNounPhrase = new Word_list_t ();
			}
			else
				plstNounPhrase->clear ();
		}
    }

    if (plstNounPhrase->size () > 1)
		_lstNgramList.push_back (plstNounPhrase);
	else
		delete plstNounPhrase;
}



//-----------------------------------------------------
void Utils::DeleteNgramList (Ngram_list_t  &_lstNgramList)
{
    ITERATE_COLLECTION (Ngram_list_t, _lstNgramList, iteNgram) 
		delete (*iteNgram);
 
    _lstNgramList.clear ();
}



//-----------------------------------------------------
void Utils::StemWord (String &_rStem, const char* _zWord) 
{
    int iLength = strlen (_zWord);
    char* pStem = new char [iLength + 2];
    strcpy (pStem, _zWord);
    PorterStemmer::Stem (pStem, iLength);
    _rStem = pStem;   
    delete[] pStem;
}



//-----------------------------------------------------
void Utils::WordListToTagList (Unigram_list_t &_lstNgramTags, Word_dq_t& _dqWords, long _lWords)
{
	long lLength = _dqWords.size ();
	if ((-1 == _lWords) || (_lWords > lLength))
		_lWords = lLength;

	for (long i = (lLength - _lWords); i < lLength; ++ i)
		_lstNgramTags.push_back (_dqWords [i]->s_POSTag);
}



//-----------------------------------------------------
void Utils::WordListToTagList (Unigram_list_t &_lstNgramTags, Word_list_t* _lstWords) 
{
    ITERATE_COLLECTION_POINTER (Word_list_t, _lstWords, iteWord)
		_lstNgramTags.push_back ((*iteWord)->s_POSTag);
}



//-----------------------------------------------------
void Utils::WordListToNgramString (String& _sNgram, Word_list_t* _plstWords, Word* _pWord) 
{
	Word_list_t::iterator	ite = _plstWords->begin ();
	Word_list_t::iterator	ite_end = _plstWords->end ();
	if (ite == ite_end)
	{
		if (NULL != _pWord)
			_sNgram << ' ' << _pWord->s_Word;
		return;
	}

	_sNgram << (*ite)->s_Word;
	++ ite;

	for (; ite != ite_end; ++ ite)
		_sNgram << ' ' << (*ite)->s_Word;

	if (NULL != _pWord)
		_sNgram << ' ' << _pWord->s_Word;
}



//-----------------------------------------------------
void Utils::WordListToNgramString (String& _sNgram, Word_dq_t* _pdqWords, Word* _pWord) 
{
	Word_dq_t::iterator	ite = _pdqWords->begin ();
	Word_dq_t::iterator	ite_end = _pdqWords->end ();
	if (ite == ite_end)
	{
		if (NULL != _pWord)
			_sNgram << ' ' << _pWord->s_Word;
		return;
	}

	_sNgram << (*ite)->s_Word;
	++ ite;

	for (; ite != ite_end; ++ ite)
		_sNgram << ' ' << (*ite)->s_Word;

	if (NULL != _pWord)
		_sNgram << ' ' << _pWord->s_Word;
}



//-----------------------------------------------------
int Utils::GetTitleWordLength (String _sTitle) 
{
    Unigram_list_t lstTitleWords;
    // TODO: Improve this
    _sTitle.Split (lstTitleWords);
    return lstTitleWords.size ();
}



//-----------------------------------------------------
void Utils::GetInOrderSectionList (Section_list_t& _lstSections,
									Section* _pSubRootSection) 
{    
    _lstSections.push_back (_pSubRootSection);
    ITERATE_COLLECTION (Section_list_t, _pSubRootSection->lst_Children, iteChildSection)
		Utils::GetInOrderSectionList (_lstSections, *iteChildSection);
}


//-----------------------------------------------------
void Utils::GetInOrderSectionList (BookSection_list_t& _lstSections, 
									BookSection* _pSubRootSection) 
{    
    _lstSections.push_back (_pSubRootSection);
    ITERATE_COLLECTION (BookSection_list_t, _pSubRootSection->lst_Children, iteChildSection)
		Utils::GetInOrderSectionList (_lstSections, *iteChildSection);
}



//-----------------------------------------------------
void Utils::SectionsAtDepth (Section_dq_t& _dqSections, Section* _pRoot, int _iDepth)
{
	ITERATE_COLLECTION (Section_list_t, _pRoot->lst_Children, ite)
	{
		Section* pSection = *ite;
		if (0 >= _iDepth)
			_dqSections.push_back (pSection);
		else
			SectionsAtDepth (_dqSections, pSection, _iDepth - 1);
	}
}


//-----------------------------------------------------
void Utils::SectionsAtDepth (BookSection_dq_t& _dqSections,
							 BookSection* _pRoot,
							 int _iDepth)
{
	ITERATE_COLLECTION (BookSection_list_t, _pRoot->lst_Children, ite)
	{
		BookSection* pSection = *ite;
		if (0 >= _iDepth)
			_dqSections.push_back (pSection);
		else
			SectionsAtDepth (_dqSections, pSection, _iDepth - 1);
	}
}



//-----------------------------------------------------
int Utils::SetIntersectionSize (Unigram_set_t &_setOne, Unigram_set_t &_setTwo) 
{
    int iMatches = 0;
    ITERATE_COLLECTION (Unigram_set_t, _setOne, ite) 
	{
		if (_setTwo.find (*ite) != _setTwo.end())
			iMatches += 1;
    }
    return iMatches;
}

