#include "Word.h"
#include <sys/types.h>
#include <regex.h>
#include <iostream>
#include <fstream>
#include <assert.h>
#include "PorterStemmer.h"
#include "Config.h"
#include "Utils.h"
#include <algorithm>

using namespace std;


Unigram_set_t	Word::set_Stopwords;
Unigram_set_t	Word::set_AuxiliaryWords;
Word*			Word::p_StartWord = NULL;
Word*			Word::p_EndWord = NULL;


//======================================================
Word::Word (const char* _zWord)
{
    s_Word = _zWord;
    Utils::StemWord (s_Stem, _zWord);

    b_IsStopWord = Word::IsStopWord (s_Word);
    b_IsAuxiliaryWord = Word::IsAuxiliaryWord (s_Word);
    b_IsContentWord = ((false == b_IsStopWord) && (false == b_IsAuxiliaryWord));
    b_StartWord = false;
    b_EndWord = false;
}

//======================================================
bool Word::IsStopWord (const String &_sWord) 
{
    if (Word::set_Stopwords.empty ())
	Word::LoadStopwords ();

    return (Word::set_Stopwords.find (_sWord) != Word::set_Stopwords.end ());
}

//======================================================
bool Word::IsAuxiliaryWord (const String &_sWord) 
{
    if (Word::set_AuxiliaryWords.empty ())
	Word::LoadAuxiliaryWords ();

    return (Word::set_AuxiliaryWords.find (_sWord) != Word::set_AuxiliaryWords.end ());
}

//======================================================
void Word::LoadStopwords () 
{
    if (false == Word::set_Stopwords.empty ())
	return;
    
    ifstream file (Config::Get ("STOPWORDS_PATH"));
    if (false == file.is_open ()) 
	{
		cerr << "Failed to open file '" 
			 << Config::Get ("STOPWORDS_PATH") 
			 << "' to read tagged text." 
			 << endl;
		return;
    }

    while (false == file.eof ()) 
	{
		String sLine;
		getline (file, sLine);
		Word::set_Stopwords.insert (sLine);
    }
}

//======================================================
void Word::LoadAuxiliaryWords () 
{
    if (false == Word::set_AuxiliaryWords.empty ())
		return;
    
    ifstream file (Config::Get ("AUXILIARY_WORDS_PATH"));
    if (false == file.is_open ()) 
	{
		cerr << "Failed to open file '" 
			 << Config::Get ("AUXILIARY_WORDS_PATH") 
			 << "' to read tagged text." 
			 << endl;
		return;
    }

    while (false == file.eof ()) 
	{
		String sLine;
		getline (file, sLine);
		Word::set_AuxiliaryWords.insert (sLine);
    }
}

//======================================================
Word* Word::GetStartWord (void) 
{
    if (p_StartWord == NULL) 
    {
        p_StartWord = new Word ("#start#");
        p_StartWord->s_POSTag = "#start#";

		p_StartWord->b_StartWord = true;

		p_StartWord->b_EndWord = false;
		p_StartWord->b_IsStopWord = false;
		p_StartWord->b_IsAuxiliaryWord = false;
		p_StartWord->b_IsContentWord = false;
    }
    return p_StartWord;
}


//======================================================
Word* Word::GetEndWord (void) 
{
    if (p_EndWord == NULL) 
    {
        p_EndWord = new Word ("#end#");
        p_EndWord->s_POSTag = "#end#";

		p_EndWord->b_EndWord = true;

		p_EndWord->b_StartWord = false;
		p_EndWord->b_IsStopWord = false;
		p_EndWord->b_IsAuxiliaryWord = false;
		p_EndWord->b_IsContentWord = false;
    }
    return p_EndWord;
}

//======================================================
bool Word::operator== (const Word& _rRight) const
{
    return ((_rRight.s_Word == s_Word) && (_rRight.s_POSTag == s_POSTag));
}

bool Word::operator!= (const Word& _rRight) const
{
    return ((_rRight.s_Word != s_Word) || (_rRight.s_POSTag != s_POSTag));
}


//======================================================
String Word::GetWordText (Word& _rWord)
{
	return _rWord.s_Word;
}

String Word::GetTagText (Word& _rWord)
{
	return _rWord.s_POSTag;
}

String Word::GetWordTagText (Word& _rWord)
{
	return (_rWord.s_Word + '_' + _rWord.s_POSTag);
}


//======================================================
ostream& operator<< (ostream& _ofs, const Word& _rWord)
{
	_ofs << _rWord.s_Word << '/' << _rWord.s_POSTag << ' ';
	return _ofs;
}

ostream& operator<< (ostream& _ofs, const Word_dq_t& _dqWords)
{
	ITERATE_COLLECTION (Word_dq_t, ((Word_dq_t&)_dqWords), ite)
	{
		Word* pWord = *ite;
		_ofs << pWord->s_Word << '/' << pWord->s_POSTag << ' ';
	}
	return _ofs;
}

ostream& operator<< (ostream& _ofs, const Word_list_t& _lstWords)
{
	ITERATE_COLLECTION (Word_list_t, ((Word_list_t&)_lstWords), ite)
	{
		Word* pWord = *ite;
		_ofs << pWord->s_Word << '/' << pWord->s_POSTag << ' ';
	}
	return _ofs;
}


//======================================================
Sentence::Sentence (void)
{
}

//======================================================
Sentence::Sentence (const char* _zText)
{
	s_Text = _zText;
	
	Utils::CleanText (s_Text);
	Utils::TokenizeText (s_Text);

	list <String>	lstTokens;
	Utils::SplitIntoWords (s_Text, lstTokens);

	list <String>::iterator	ite;
	for (ite = lstTokens.begin ();
		ite != lstTokens.end ();
		++ ite)
	{
		ite->LowerCase();

	    Word* pWord = new Word (*ite);
	    lst_Words.push_back (pWord);
	    set_Stems.insert (pWord->s_Stem);
	}
}

//======================================================
void Sentence::AddWord (Word* _pWord)
{
	lst_Words.push_back (_pWord);
	set_Stems.insert (_pWord->s_Stem);
}

//======================================================
String Sentence::GetTaggedText (void)
{
	String sTaggedText;
	String sLowerText = s_Text;

	sLowerText.LowerCase ();

	Unigram_list_t lstLowerTokens;
	sLowerText.Split (lstLowerTokens);

	Word_list_t::iterator iteWord = lst_Words.begin ();
	unsigned int iTokenIndex = 0;
	ITERATE_COLLECTION (Unigram_list_t, lstLowerTokens, iteLowerToken)  
	{
	    String sLowerToken (*iteLowerToken);
		String sToken (sLowerToken);

		if ((sToken == "-lrb-") || (sToken == "-rrb-") ||
			(sToken == "-lsb-") || (sToken == "-rsb-") ||
			(sToken == "-lcb-") || (sToken == "-rcb-"))
			sToken.UpperCase ();

	    if (0 != iTokenIndex)
			sTaggedText << ' ';
	    sTaggedText << sToken;
	    
		Word* pWord = *iteWord;
		if (iteWord != lst_Words.end () && sLowerToken == pWord->s_Word) 
		{
			sTaggedText << "_" << pWord->s_POSTag;
			++iteWord;
		}
	    
	    ++iTokenIndex;
	}

	return sTaggedText;
}


//======================================================
Word_list_t& Sentence::GetWords (void)  
{ 
	return lst_Words; 
}


//======================================================
const char* Sentence::GetText (void) 
{ 
	return s_Text; 
}


