#ifndef __TITLE_GEN_WORD_
#define __TITLE_GEN_WORD_

#include <list>
#include <deque>
#include <map>
#include <set>
#include "hash_map.hpp"
#include "nlp_string.h"
#include <iostream>
#include <fstream>


using namespace std;


class Word;
class Sentence;

typedef list <Sentence*>			Sentence_list_t;
typedef list <String>				Unigram_list_t;
typedef list <Word*>				Word_list_t;
typedef deque <Word*>				Word_dq_t;
typedef list <Word_list_t*>			Ngram_list_t;
typedef set <String>				Unigram_set_t;
typedef hash_map <String, long>		Unigram_to_Count_map_t;
typedef hash_map <String, double>	FrequencyRank_By_Stem_map_t;
typedef hash_map <String, long>		Ngram_to_Count_map_t;

typedef String (*WordRenderer_t)(Word& _rWord);



//======================================================
class Word
{
	private:
		static void LoadStopwords ();
		static void LoadAuxiliaryWords ();
		static bool IsStopWord (const String &_sWord);
		static bool IsAuxiliaryWord (const String &_sWord);

	public:
		static Unigram_set_t	set_Stopwords;
		static Unigram_set_t	set_AuxiliaryWords;
		static Word*			p_StartWord;
		static Word*			p_EndWord;
    
		String	s_Word;
		String	s_Stem;
		String	s_POSTag;
		bool	b_IsStopWord;
		bool	b_IsAuxiliaryWord;
		bool	b_IsContentWord;
		bool	b_StartWord;
		bool	b_EndWord;

		Word (const char* _zWord);

		bool operator== (const Word& _w) const;
		bool operator!= (const Word& _w) const;

		static Word* GetStartWord (void);
		static Word* GetEndWord (void);
		static String GetWordText (Word& _rWord);
		static String GetWordTagText (Word& _rWord);
		static String GetTagText (Word& _rWord);
};

ostream& operator<< (ostream& _ofs, const Word& _rWord);
ostream& operator<< (ostream& _ofs, const Word_dq_t& _dqWords);
ostream& operator<< (ostream& _ofs, const Word_list_t& _lstWords);



//======================================================
class Sentence
{
	private:
		// use parse tree here instead...
		
	public:
		Word_list_t		lst_Words;
		Unigram_set_t	set_Stems;
		String 			s_Text;

		Sentence (void);
		Sentence (const char* _zText);
		~Sentence (void);

		void AddWord (Word* _pWord);
		Word_list_t& GetWords (void);
		const char* GetText (void);
		String GetTaggedText (void);
};



#endif
