#ifndef __NLP_LIBRARY_TYPES_STRING__
#define __NLP_LIBRARY_TYPES_STRING__ 

#include <iostream>
#include <string> 
#include <list>
using namespace std;



// ---------------------------------------------------------------------
class SubMatch
{
    public:
        long l_SourceOffset;
        long l_TargetOffset;
        long l_Length;
        char* p_ReplaceText;
};

class Match
{
    public:
        long l_StartOffset;
        long l_EndOffset;
        long l_Shift;
        SubMatch* p_SubMatches;
};



// ---------------------------------------------------------------------
class String : public string
{
public:
	String (void);
	String (const char* _zText);
	String (const string& _sText);
	String (const String& _sText);
	String (const list <String>& _rList);
	String (const list <string>& _rList);

	int Compare (const String& _rString);
	int Compare (const char* _zValue);
	int Compare (const char _cValue);
	
	int CaseCompare (const String& _rString);//..compare string ignoring case.
	int CaseCompare (const char* _zValue);   //..compare string ignoring case.
	int CaseCompare (const char _cValue);   //..compare string ignoring case.
	
    // ---------------------------------------------------
    void Capitalize (void);
    void LowerCase (void);
    void UpperCase (void);
    void TitleCase (void);
    void Replace (const char* _zSearch, const char* _zReplace);
    void ReplacePattern (const char* _zPattern, const char* _zReplace);

    void Join (const list <String>& _rList, char _cSeparator = ' ');
    void Join (const list <string>& _rList, char _cSeparator = ' ');
    void Split (list <String>& _rList, char _cSeparator = '\0', int _iTotalSplits = 0);
    void SplitByString (list <String>& _rList, const char* _zSeparator, int _iTotalSplits = 0);
    void SplitByRegexp (list <String>& _rList, const char* _zPattern, int _iTotalSplits = 0);
    void SplitBySeparatorList (list <String>& _rList, const char* _zSeparatorList, int _iTotalSplits = 0);
    void SplitLines (list <String>& _rList, char _cSeparator = '\0', int _iTotalSplits = 0);

    long Count (const char* _zSubString, long _lStart = 0, long _lEnd = -1);
    size_type Find (const char* _zSubString, long _lStart = 0, long _lEnd = -1);
    size_type ReverseFind (const char* _zSubString, long _lStart = 0, long _lEnd = -1);

    bool IsAlNum (void);
    bool IsAlpha (void);
    bool IsDigit (void);
    bool IsSpace (void);
    bool IsLower (void);
    bool IsUpper (void);
    bool IsTitle (void);

    // operators -----------------------------------------
	operator bool (void) const;
	operator int (void) const;
	operator long (void) const;
	operator double (void) const;
	operator const char* (void) const;
	operator string (void) const;

	bool operator== (const bool _bValue) const;
	bool operator== (const char _cValue) const;
	bool operator== (const int _iValue) const;
	bool operator== (const long _lValue) const;
	bool operator== (const double _dValue) const;
	bool operator== (const char* _zValue) const;
	bool operator== (const string& _sValue) const;

	bool operator!= (const bool _bValue) const;
	bool operator!= (const char _cValue) const;
	bool operator!= (const int _iValue) const;
	bool operator!= (const long _lValue) const;
	bool operator!= (const double _dValue) const;
	bool operator!= (const char* _zValue) const;
	bool operator!= (const string& _sValue) const;

	String& operator= (bool _bValue);
	String& operator= (char _cValue);
	String& operator= (int _iValue);
	String& operator= (long _lValue);
	String& operator= (double _dValue);
	String& operator= (const char* _zValue);
	String& operator= (const String& _sValue);
	String& operator= (const string& _sValue);

	String& operator+= (int _iValue);
	String& operator+= (char _cValue);
	String& operator+= (long _lValue);
	String& operator+= (double _dValue);
	String& operator+= (const char* _zValue);
	String& operator+= (const string& _sValue);

	String operator+ (int _iValue);
	String operator+ (char _cValue);
	String operator+ (long _lValue);
	String operator+ (double _dValue);
	String operator+ (const char* _zValue);
	String operator+ (const string& _sValue);

	bool operator< (const bool _bValue) const;
	bool operator< (const char _cValue) const;
	bool operator< (const int _iValue) const;
	bool operator< (const long _lValue) const;
	bool operator< (const double _dValue) const;
	bool operator< (const char* _zValue) const;
	bool operator< (const string& _sValue) const;

	bool operator> (const bool _bValue) const;
	bool operator> (const char _cValue) const;
	bool operator> (const int _iValue) const;
	bool operator> (const long _lValue) const;
	bool operator> (const double _dValue) const;
	bool operator> (const char* _zValue) const;
	bool operator> (const string& _sValue) const;

	bool operator<= (const bool _bValue) const;
	bool operator<= (const char _cValue) const;
	bool operator<= (const int _iValue) const;
	bool operator<= (const long _lValue) const;
	bool operator<= (const double _dValue) const;
	bool operator<= (const char* _zValue) const;
	bool operator<= (const string& _sValue) const;

	bool operator>= (const bool _bValue) const;
	bool operator>= (const char _cValue) const;
	bool operator>= (const int _iValue) const;
	bool operator>= (const long _lValue) const;
	bool operator>= (const double _dValue) const;
	bool operator>= (const char* _zValue) const;
	bool operator>= (const string& _sValue) const;
};


String& operator<< (String& _rLeft, const String& _rRight);
String& operator<< (String& _rLeft, const string& _rRight);
String& operator<< (String& _rLeft, const char* _zRight);
String& operator<< (String& _rLeft, const char _cRight);
String& operator<< (String& _rLeft, const bool _bRight);
String& operator<< (String& _rLeft, const int _iRight);
String& operator<< (String& _rLeft, const long _lRight);
String& operator<< (String& _rLeft, const double _dRight);
String& operator<< (String& _rLeft, const list <String>& _rList);
String& operator<< (String& _rLeft, const list <string>& _rList);

ostream& operator<< (ostream& _rStream, const list <String>& _rList);
ostream& operator<< (ostream& _rStream, const list <string>& _rList);


#endif
